#include <cstdlib>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip> 
#include <set>
#include <gsl/gsl_math.h>

#include "IOUtilities.h"





using namespace std;

void CTrainingConfig::Print() {
	cout << name << endl;
	cout << "size: " << index.size() << endl;
	PrintVctr(index);
}
int CTrainingConfig::size() {
	return((int)index.size());
}

bool SaveData(const bool bbase, const INT_VCTR_1D& isBase, const DBL_VCTR_2D& Xtild, const INT_VCTR_3D& X, const string& File)
{
	if (Xtild.size() == 0)
	{
	  cout << "No covariates in Xtild -> Data not saved - saveData" << endl;
	  return false;
	  //cerr << "Uninitialized Xtild - SaveData" << endl;
	  //exit(EXIT_FAILURE);
	}

	if (X.size() == 0)
	{
		cerr << "Uninitialized X - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	if (bbase && (isBase.size()==0) )
	{
		cerr << "Uninitialized isBase with bbase = true - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	if ( Xtild.size() < X.size() )
	{
		cerr << "Mismatched Xtild and X - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	int i, a, l,b;

	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(Xtild.size());
	int B = static_cast<int>(Xtild[0].size());
	int N = static_cast<int>(X.size());
	int A = static_cast<int>(X[0].size());
	int L = static_cast<int>(X[0][0].size());


	// cout << "SaveData - N="<<N<<" A="<<A<<" L="<<L<<" M="<<M<<" B="<< B <<" bbase="<<bbase<<endl;

	for (i = 0; i < M; i++)
	{
		for (a = 0; a < A; a++)
		{
			if (bbase)
			{
				ofs << isBase[i] << "\t";
			}
			for (b=1;b<B;b++)
			{
				ofs << Xtild[i][b] << "\t";
			}
			if (i < N)
			{
				for (l = 0; l < L; l++)
				{
					ofs << X[i][a][l] << "\t";
				}
			}
			else
			{
				for (l = 0; l < L; l++)
				{
					ofs << 0 << "\t";
				}
			}
			ofs << endl;
		}
	}
	
	ofs.close();
	
	return true;
}

//! Loading Genotype and Covariates Data and other Informations contained in File
bool LoadData(const bool bbase, INT_VCTR_1D& isBase,DBL_VCTR_2D& C, DBL_VCTR_2D& Xtild, INT_VCTR_2D& XQual, INT_VCTR_2D& LevelPopSize, INT_VCTR_1D& nQualLevels, INT_VCTR_3D& X, INT_VCTR_1D& recAllele, int ER, int EC, const string& File, const bool& specialFormat, const int& nbQuantVar, const int& nb_var_regr, const bool& isRecAllele, const int& trendDegree)
{
	
	if (C.size() == 0)
	{
		cerr << "Uninitialized C - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (X.size() == 0)
	{
		cerr << "Uninitialized X - LoadData" << endl;
		exit(EXIT_FAILURE);
	}


	int i, a, l, b, var;
	int begin_quantitative=1;

	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cout << "Cannot open file " << File << " - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(C.size()); // N + dummy individuals
	int N = static_cast<int>(X.size());
	int A = static_cast<int>(X[0].size());
	int L = static_cast<int>(X[0][0].size());
	int nbQualVar = static_cast<int>(XQual[0].size()); // Number of Qualitative Variables

	

	
	double x,y;
	int trendSize,XQuantCol;
	//Degree of trend
	// The intercept is NOT taken into account in trendSize
	if (trendDegree == -1 || nb_var_regr == 0)
	  {
	    trendSize=0;
	  }
	else 
	  {
	    if (nb_var_regr==1)
	      {
		trendSize = trendDegree;
	      }
	    else
	      {
		trendSize = (trendDegree +1)*(trendDegree +2)/2 - 1;
	      }
	  }
	
	if (nbQuantVar==0 && nbQualVar==0 && (trendDegree==-1||trendDegree==0)) // no covariates -> no intercept either
	  {
	    XQuantCol=0;
	  }
	else
	  {
	    // 1 for the intercept + trend + nbQuantVar quantitative covariates 
	    XQuantCol = 1 + trendSize + nbQuantVar;
	  }
	
	DBL_VCTR_2D XQuant; // intercept + quanti + trend
	if (XQuantCol != 0)
	  {
	    InitVctr2D(XQuant, N, XQuantCol, 1.0);
	  }

	// Load and discard the extra rows.
	char extraRow[10240];
	
	for (int er = 0; er < ER; er++)
	{
		ifs.getline(extraRow, 10240);
	}

	int recAl = 1;
	if (isRecAllele)
	{
		for (l = 0; l < L; l ++)
		{
			ifs >> recAl;
			recAllele[l] = recAl;
		}
	}

	if (!specialFormat)
	{
		for (i = 0; i < M; i++)
		{
			for (a = 0; a < A; a++)
			{
				// Load and discard the extra columns.
				string discarded;
				for (int ec = 0; ec < EC; ec++)
				{
					ifs >> discarded;
				}

				//Stock base information
				if (bbase)
				{
					ifs >> isBase[i];
				}

				// Qualitative Variables
				if (nbQualVar > 0)
				{
				  for (var = 0; var < nbQualVar; var++)
				    {
				      ifs >> XQual[i][var];
				    }
				}
				
				int begin_coordinates = begin_quantitative + nbQuantVar;
							
				// Quantitative Variable
				for (b = begin_quantitative; b < begin_coordinates; b++)
				{
					ifs >> XQuant[i][b];
				}
				
				// Spatial Coordinates (if given in the datafile)

				if (trendDegree != -1) 
				{  
					ifs >> C[i][0];
					ifs >> C[i][1];

					x = C[i][0];
					y = C[i][1];
										
					if (nb_var_regr == 1)
					{
					  // We keep only y (ignoring x column)
						for (b = begin_coordinates; b < XQuantCol; b ++)
						{
						  XQuant[i][b] = gsl_pow_int(y,b-begin_coordinates+1);
						}
					}

					if (nb_var_regr == 2)
					{
					 
						if (trendDegree > 0)
						{
							XQuant[i][begin_coordinates] = x;
							XQuant[i][begin_coordinates+1] = y;
						
							if (trendDegree > 1)
							{
								XQuant[i][begin_coordinates+2] = x*x;
								XQuant[i][begin_coordinates+3] = y*y;
								XQuant[i][begin_coordinates+4] = x*y;
								if (trendDegree > 2)
								{
									XQuant[i][begin_coordinates+5] = x*x*x;
									XQuant[i][begin_coordinates+6] = y*y*y;
									XQuant[i][begin_coordinates+7] = x*x*y;
									XQuant[i][begin_coordinates+8] = x*y*y;
								}
							}
						}
					}
				}
				// Genotype Data
				if (i < N)
				{
					for (l = 0; l < L; l++)
					{
						ifs >> X[i][a][l];
					}
				}
				else
				{
					for (l = 0; l < L; l++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}
	if (specialFormat)
	{
		for (i = 0; i < M; i ++)
		{
			// Load and discard the extra columns.
			string discarded;
			for (int ec = 0; ec < EC; ec++)
			{
				ifs >> discarded;
			}
			// Stock base information
			if (bbase)
			{
				ifs >> isBase[i];
			}

			// Qualitative Variables
			if (nbQualVar > 0)
			  {
			    for (var = 0; var < nbQualVar; var++)
			      {
				ifs >> XQual[i][var];
			      }
			  }
						
			int begin_coordinates=begin_quantitative + nbQuantVar;
			// Quantitative Variable
			for (b = begin_quantitative; b < begin_coordinates; b++)
			{
				ifs >> XQuant[i][b];
			}

			// Spatial Coordinates (if given in the datafile)
			if (trendDegree != -1) 
			{
				ifs >> C[i][0] >> C[i][1];
				x = C[i][0];
				y = C[i][1];
				
				if (nb_var_regr == 1)
				{
					// We keep only y (ignoring leftmost column)
					for (b = begin_coordinates; b < XQuantCol; b ++)
					{
						XQuant[i][b] = gsl_pow_int(y,b-begin_coordinates+1);
					}
				}
				
				if (nb_var_regr == 2)
				{
					
				  if (trendDegree > 0) 
					{
						XQuant[i][begin_coordinates] = x;
						XQuant[i][begin_coordinates+1] = y;
						if (trendDegree > 1) 
						{
							XQuant[i][begin_coordinates+2] = x*x;
							XQuant[i][begin_coordinates+3] = y*y;
							XQuant[i][begin_coordinates+4] = x*y;
							
							if (trendDegree > 2) 
							{
								XQuant[i][begin_coordinates+5] = x*x*x;
								XQuant[i][begin_coordinates+7] = y*y*y;
								XQuant[i][begin_coordinates+8] = x*x*y;
								XQuant[i][begin_coordinates+9] = x*y*y;
							}
						}
					}
				}
			}
			

			//Genotype Data
			if (i < N)
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> X[i][a][l];
					}
				}
			}
			else
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}

	ifs.close();


	if (nbQualVar <= 0) 
	  {
	    Xtild = XQuant;
	   
	    return true;
	  }
	else
	  {
	    bool bQual = TransformQualData(XQual, XQuant, Xtild, nQualLevels, LevelPopSize);
	    return bQual;
	  }
	
}


bool TransformQualData(INT_VCTR_2D& XQual, DBL_VCTR_2D& XQuant, DBL_VCTR_2D& Xtild, INT_VCTR_1D& nQualLevels, INT_VCTR_2D& LevelPopSize) 
{
  
  
  int N =  static_cast<int>(XQuant.size());
  int nbQualVar =  static_cast<int>(XQual[0].size());
  int nbQuantColumns  =  static_cast<int>(XQuant[0].size()); // contains 1 column for intercept
  int nbQualColumns = 0;

  int begin_quantitative,nextCol,currCol;
  // loop index
  int var,i,b,lev;

  vector< set< int > > levelS;
  set< int > emptyS;

  for (var = 0; var < nbQualVar; var++)
    {
      levelS.push_back(emptyS);
    }
  for (var = 0; var < nbQualVar; var++)
    {
      for (i = 0; i < N; i++)
	{
	  levelS[var].insert(XQual[i][var]);
	}
      nQualLevels[var] = static_cast<int> (levelS[var].size());
      nbQualColumns +=  static_cast<int> (levelS[var].size()) - 1;
    }

 

  InitVctr2D(LevelPopSize,nbQualVar,nQualLevels,0);
  InitVctr2D(Xtild,N, nbQualColumns + nbQuantColumns, 0.0);
  
  for (i = 0; i < N; i++)
    {
      Xtild[i][0]=1.0; //Intercept
     
      nextCol=1;
      currCol=1;
      
      for (var = 0; var < nbQualVar; var++)
	{
	  nextCol += nQualLevels[var] - 1;
	  set< int >::const_iterator csiB = levelS[var].begin();
	  set< int >::const_iterator csiE = levelS[var].end();
	  set< int >::const_iterator csi,it;

	  
	  it=levelS[var].find(XQual[i][var]);
	  if (it == csiB)
	    {
	      LevelPopSize[var][0]++;
	      for (b = currCol; b < nextCol; b++)
		{
		  Xtild[i][b]=-1;
		}
	      XQual[i][var]=0; 
	    }
	  else
	    {
	      lev = distance(csiB,it); //should give an index ?
	      LevelPopSize[var][lev]++;
	      Xtild[i][currCol+lev-1]=1; //other columns stay at 0.0
	      XQual[i][var]=lev;
	    }
	  
	  /* if (XQual[i] ==  levelS[var][*csiB] )
	    {
	      LevelPopSize[var][0]++;
	      for (b = currCol; b < nextCol; b++)
		{
		  Xtild[i][b]=-1;
		}
	    }
	  else
	    {
	       for (lev = 1; lev < nQualLevels[var]; lev++)
	       
		{
		  if (XQual[i][var] ==  levelS[var][lev])
		    {
		      LevelPopSize[var][lev]++;
		      Xtild[i][currCol+lev-1]=1; //other columns stay at 0.0
		    }
		}
	    }*/
	  currCol = nextCol;
	}
      
      begin_quantitative = nextCol;
      for (int b = begin_quantitative; b <  static_cast<int> (Xtild[0].size()) ; b++)
	{
	  Xtild[i][b]=XQuant[i][b-begin_quantitative+1]; //XQuant[][1] corresponds to Intercept -> not needed 
	}
    }
 
 
 return true;
 
}


bool SaveCF(const INT_VCTR_1D& z, const DBL_VCTR_3D& p, const string& File)
{
	if (z.size() == 0)
	{
		cerr << "Uninitialized z - SaveCF" << endl;
		return false;
	}

	if (p.size() == 0)
	{
		cerr << "Uninitialized p - SaveCF" << endl;
		return false;
	}

	int i, k, l, j;

	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveCF" << endl;
		return false;
	}

	// K, L, J
	int K = static_cast<int>(p.size());
	int L = static_cast<int>(p[0].size());
	INT_VCTR_1D J(L, 0);
	for (l = 0; l < L; l++)
	{
		J[l] = static_cast<int>(p[0][l].size());
	}
	ofs << K << ' ';
	ofs << L << ' ';
	for (l = 0; l < L; l++)
	{
		ofs << J[l] << ' ';
	}

	// N
	int N = static_cast<int>(z.size());
	ofs << N << ' ';

	// p
	for (k = 0; k < K; k++)
	{
		for (l = 0; l < L; l++)
		{
			for (j = 0; j < J[l]; j++)
			{
				ofs << p[k][l][j] << ' ';
			}
		}
	}

	// z
	for (i = 0; i < N; i++)
	{
		ofs << z[i] << ' ';
	}

	ofs.close();

	return true;
}

bool LoadCF(INT_VCTR_1D& z, DBL_VCTR_3D& p, const string& File)
{
	if (z.size() == 0)
	{
		cerr << "Uninitialized z - LoadCF" << endl;
		return false;
	}

	if (p.size() == 0)
	{
		cerr << "Uninitialized p - LoadCF" << endl;
		return false;
	}

	int i, k, l, j;

	// K, L, J
	int K = static_cast<int>(p.size());
	int L = static_cast<int>(p[0].size());
	INT_VCTR_1D J(L, 0);
	for (l = 0; l < L; l++)
	{
		J[l] = static_cast<int>(p[0][l].size());
	}

	// N
	int N = static_cast<int>(z.size());

	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cout << "Cannot open file " << File << " - LoadCF" << endl;
		return false;
	}

	// KF, LF, JF
	int KF;
	int LF;
	INT_VCTR_1D JF;
	ifs >> KF;
	ifs >> LF;
	for (l = 0; l < LF; l++)
	{
		JF.push_back(0);
		ifs >> JF[l];
	}

	// NF
	int NF;
	ifs >> NF;
	if (KF != K || LF != L || JF != J || NF != N)
	{
		ifs.close();

		cerr << "CF file conflicts with command-line options" << " - LoadCF" << endl;
		return false;
	}

	for (k = 0; k < KF; k++)
	{
		for (l = 0; l < LF; l++)
		{
			for (j = 0; j < JF[l]; j++)
			{
				ifs >> p[k][l][j];
			}
		}
	}

	for (i = 0; i < NF; i++)
	{
		ifs >> z[i];
	}

	ifs.close();

	return true;
}


void SaveToMyFile(const DBL_VCTR_2D& V, const string& File){
	int i,j;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveToMyFile" << endl;
		exit(1);
	}
	for (i=0;i<(int)V.size();i++){
		for (j=0;j<(int)V[0].size();j++){
			ofs<<V[i][j]<<"\t";
		}
		ofs<<endl;
	}
	ofs.close();
}

void SaveToMyFile(const DBL_VCTR_1D& V, const string& File){
	int i;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveToMyFile" << endl;
		exit(1);
	}
	for (i=0;i<(int)V.size();i++){
		ofs<<V[i]<<endl;
	}
	ofs.close();
}

void SaveToMyFile(const INT_VCTR_2D& V, const string& File){
	int i,j;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveToMyFile" << endl;
		exit(1);
	}
	for (i=0;i<(int)V.size();i++){
		for (j=0;j<(int)V[0].size();j++){
			ofs<<V[i][j]<<"\t";
		}
		ofs<<endl;
	}
	ofs.close();
}

void SaveToMyFile(const INT_VCTR_1D& V, const string& File){
	int i;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveToMyFile" << endl;
		exit(1);
	}
	for (i=0;i<(int)V.size();i++){
		ofs<<V[i]<<endl;
	}
	ofs.close();
}


void SaveBetaToMyFile(const DBL_VCTR_3D& Beta, const string& File, const int BurnInPeriod){
	int K=(int)Beta.size();
	int P=(int)Beta[0].size();
	int TotalSweep=(int)Beta[0][0].size();
	int KeptSweep = TotalSweep - BurnInPeriod;
	int s,k,p;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveBetaToMyFile" << endl;
		return;
	}
	ofs << "* Beta Posterior Distribution * (Values are sampled after burn-in, with a minimum step of 10 sweeps, and no more than 1000 sweeps of the MCMC are recorded.)" << endl;
	ofs << "Nb.clusters:\t" <<  K+1 <<  "\t Beta.length:\t" << P << endl;

	// Values are sampled each 'step' sweeps, with 'step'>= 10
	// No more than 1000 sweeps are recorded
	if (floor(KeptSweep/10) <= 1000) {
	  for (s=BurnInPeriod; s < TotalSweep; s+=10){
	    for (k=0; k<K; k++){
	      for (p=0; p<P; p++){
		//	cout << Beta[cycle][k][p]<<"\t";
		ofs << Beta[k][p][s] << "\t";
	      }
	    }
	    ofs << endl;
	  }
	} else {
	  int step = floor(KeptSweep/1000);
	  for (s=BurnInPeriod; s < TotalSweep; s+=step){
	    for (k=0; k<K; k++){
	      for (p=0; p<P; p++){
		//	cout << Beta[cycle][k][p]<<"\t";
		ofs << Beta[k][p][s] << "\t";
	      }
	    }
	    ofs << endl;
	  }
	}



	ofs.close();
	cout<<"Beta Posterior Distribution saved in "<<File<<endl;
}

void SaveToMyFileClumppFormat(const DBL_VCTR_2D& V, const string& File, const bool header, const string title)
{
	int i,k;
	int N=static_cast<int>(V.size());
	int K=static_cast<int>(V[0].size());
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveToMyFileClusterFormat" << endl;
		exit(1);
	}

	if (header)
	  {
	    ofs << title << endl;
	    
	    ofs << "Sample\t";
	    for (k = 0; k < K; k++)
	      {
		ofs << "***Cluster  ";
		ofs.width(2);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << k + 1 << "***\t";
	      }
	    ofs << endl;
	  }

	for (i = 0; i < N; i++)
	{
		ofs.width(6);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << i + 1 << ":\t";
		for (k = 0; k < K; k++)
		{
			ofs.width(17);
			ofs.precision(15);
			ofs << fixed;
			ofs << V[i][k] << " ";
		}
		ofs << "\t" << 1 << endl;
	}
	ofs << endl << endl << endl;

}

void WriteInOfstreamClummpFormat(const DBL_VCTR_2D V, ofstream& ofs)
{

	int i,k;
	int N=static_cast<int>(V.size());
	int K=static_cast<int>(V[0].size());

	ofs << "Sample\t";
	for (k = 0; k < K; k++)
	{
		ofs << "***Cluster  ";
		ofs.width(2);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << k + 1 << "***\t";
	}
	ofs << endl;
	for (i = 0; i < N; i++)
	{
		ofs.width(6);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << i + 1 << ":\t";
		for (k = 0; k < K; k++)
		{
			ofs.width(17);
			ofs.precision(15);
			ofs << fixed;
			ofs << V[i][k] << " ";
		}
		ofs << "\t" << 1 << endl;
	}
	ofs << endl << endl << endl;

}


bool LoadTraining(vector<CTrainingConfig> &vTrainingConfig,vector<CTrainingConfig> &vCompTrainingConfig, bool &indivValidation, const int N, const int L, const string &File){

	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cout << "Cannot open file " << File << " - LoadTraining" << endl;
		exit(EXIT_FAILURE);
	}

	string validationMethod;
	int indexMax;
	getline(ifs,validationMethod);
	if ((validationMethod.find("individuals") != string::npos) || (validationMethod.find("Individuals") != string::npos) || (validationMethod.find("INDIVIDUALS") != string::npos))
	{
		indivValidation = true; // Validation on Individuals
		indexMax=N;
	}
	else if ((validationMethod.find("loci") != string::npos) || (validationMethod.find("Loci") != string::npos) || (validationMethod.find("LOCI") != string::npos))
	{
		indivValidation = false; // Validation on Loci
		indexMax=L;
	}
	else
	{
		cout << "The first line of the Training Configuration file must contain the Validation Method:" << endl;
		cout << "Individuals or Loci" << endl;
		exit(EXIT_FAILURE);
	}

	char tmp[1000];
	stringstream index;
	string line,indexLine;
	size_t pos;
	int i;

	while(getline(ifs,line))
	{

		pos=line.find(":");
		if (pos!=string::npos)
		{
			CTrainingConfig config;
			CTrainingConfig compConfig;
			config.name=line.substr(0,pos);
			compConfig.name="comp."+config.name;
			indexLine= line.substr(pos+1,string::npos);
			if (indexLine.empty())
			{
				cout<< "Cannot read training index for configuration "<<config.name<<endl;
				exit(EXIT_FAILURE);
			}
			else 
			{
				index.str(indexLine);
				int tmpIndex=1;
				while (!index.eof())
				{
					index >> tmp;
					for (i=tmpIndex; i<atoi(tmp); i++)
					{
						compConfig.index.push_back(i); 
					}
					config.index.push_back(atoi(tmp));
					tmpIndex=atoi(tmp)+1;
				} 
				for (i=config.index.back()+1; i<=indexMax; i++)
				{
					compConfig.index.push_back(i); 
				}

				//config.size=(config.index).size();
				vTrainingConfig.push_back(config);
				vCompTrainingConfig.push_back(compConfig);

				index.clear();
			}
		}
	}

	return true;
}





bool LoadQualInfoFromData(const int N, const int nbQualVar, const int nbQuantVar, const int nbLoci, const int A, const int ER, const int EC, const bool& specialFormat, const bool& bbase, const bool& isRecessiveAllele, const bool& hasSpatialCoord, vector< set< int > > & levelS,  const string& File )
{
  int extraLeftCol = EC;
  if (bbase) 
    extraLeftCol += 1;
  int extraRightCol = nbQuantVar;
  int nLinesIndiv; //nb of lines containing individual information (depending on the format used)
  if (hasSpatialCoord)
    extraRightCol += 2;
  if (specialFormat)
    {
      extraRightCol += nbLoci*A;
      nLinesIndiv = N;
    }
  else
    {
      extraRightCol += nbLoci;
      nLinesIndiv = N*A;
    }
  
  int extraUpRow = ER;
  if (isRecessiveAllele)
    extraUpRow += 1;


  /* string text;
     stringstream out;
     out << "init " << extraLeftCol << "  " << extraRightCol;
     text=out.str();
     SaveText(text,"log.txt");
  */

  ifstream ifs(File.c_str());
  if (!ifs.good())
    {
      cout << "Cannot open file " << File << " - LoadQualInfoFromData" << endl;
      exit(EXIT_FAILURE);
    }

  int var,i,tmp;

  set< int > emptyS;

  for (var = 0; var < nbQualVar; var++)
    {
      levelS.push_back(emptyS);
    }


  // Load and discard the extra rows.
  char extraRow[10240];
  
  for (int er = 0; er < extraUpRow; er++)
    {
      ifs.getline(extraRow, 10240);
    }
 
  for (i = 0; i < nLinesIndiv; i++)
    {
      // Load and discard the extra columns.
      string discarded;
      for (int elc = 0; elc < extraLeftCol; elc++)
	{
	  ifs >> discarded;
	}

      for (var = 0; var < nbQualVar; var++)
	{
	  ifs >> tmp;
	  levelS[var].insert(tmp);
	}
      for (int erc = 0; erc < extraRightCol; erc++)
	{
	  ifs >> discarded;
	}
     
    }
  ifs.close();
  
  /*int nbQualColumns=0;
      for (int var = 0; var < levelS.size(); var++)
	{
	  out << " var  " << var << " col " << nbQualColumns;
	  nbQualColumns +=  static_cast<int> (levelS[var].size()) - 1;
	  out << " col " << nbQualColumns << endl;
	}
	SaveText(out.str(),"log.txt");*/

  return(true);
	
}


//! Loading  Covariates Data and other Informations contained in File

bool LoadNewData(DBL_VCTR_2D& C, DBL_VCTR_2D& Xtild, int ER, int EC, const int& nbQuantVar, const bool& hasSpatialCoord,  vector< set< int > > levelS, const string& File)

{
	
  if (C.size() == 0)
    {
      cerr << "Uninitialized C - LoadNewData" << endl;
      //exit(EXIT_FAILURE);
    }
  if (Xtild.size() == 0)
    {
      cerr << "Uninitialized Xtild - LoadNewData" << endl;
      //exit(EXIT_FAILURE);
      return false;

    }



 
  ifstream ifs(File.c_str());
  if (!ifs.good())
    {
      cout << "Cannot open file " << File << " - LoadNewData" << endl;
      //      exit(EXIT_FAILURE);
      return false;

    }
  
  int N = static_cast<int>(Xtild.size()); // N 
  //int B = static_cast<int>(Xtild[0].size());

  // int M = static_cast<int>(C.size()); // N + dummy individuals
  int nbQualVar = static_cast<int>(levelS.size()); // Number of Qualitative Variables
  int nbQualColumns = 0;
  
  for (int var = 0; var < nbQualVar; var++)
    {
      nbQualColumns +=  static_cast<int> (levelS[var].size()) - 1;
    }
  int begin_quantitative = 1 + nbQualColumns;

  int i, b, var, lev, nextCol, currCol;
  double x,y;
 
  int value;

  // Load and discard the extra rows.
  char extraRow[10240];
	
  for (int er = 0; er < ER; er++)
    {
      ifs.getline(extraRow, 10240);
    }

  for (i = 0; i < N; i++)
    {
      Xtild[i][0]=1.0; // Intercept
	
      // Load and discard the extra columns.
      string discarded;
      for (int ec = 0; ec < EC; ec++)
	{
	  ifs >> discarded;
	}
      
      
      // Qualitative Variables
      if (nbQualVar > 0)
	{
	  nextCol=1;
	  currCol=1;
      
	  for (var = 0; var < nbQualVar; var++)
	    {
	      ifs >> value;
	      nextCol += levelS[var].size() - 1;
	      set< int >::const_iterator csiB = levelS[var].begin();
	      set< int >::const_iterator csiE = levelS[var].end();
	      set< int >::const_iterator csi,it;

	  
	      it=levelS[var].find(value);
	      if (it == csiB)
		{
		  for (b = currCol; b < nextCol; b++)
		    {
		      Xtild[i][b]=-1;
		    }
		}
	      else
		{
		  lev = distance(csiB,it); //should give an index ?
		  Xtild[i][currCol+lev-1]=1; //other columns stay at 0.0
		}
	      // if not found error
	      
	      currCol = nextCol;
	    }
	}
      
      int begin_coordinates = begin_quantitative + nbQuantVar;
      
      // Quantitative Variable
      for (b = begin_quantitative; b < begin_coordinates; b++)
	{
	  ifs >> Xtild[i][b];
	}
      
      // Spatial Coordinates (if given in the datafile)
      // We load the Trend of Maximal Degree (ie degree=3)
      
      if (hasSpatialCoord) 
	{  
	  ifs >> C[i][0];
	  ifs >> C[i][1];
	  
	  x = C[i][0];
	  y = C[i][1];
	  
	  
	  Xtild[i][begin_coordinates] = x;
	  Xtild[i][begin_coordinates+1] = y;
	  
	  Xtild[i][begin_coordinates+2] = x*x;
	  Xtild[i][begin_coordinates+3] = y*y;
	  Xtild[i][begin_coordinates+4] = x*y;
	  
	  Xtild[i][begin_coordinates+5] = x*x*x;
	  Xtild[i][begin_coordinates+6] = y*y*y;
	  Xtild[i][begin_coordinates+7] = x*x*y;
	  Xtild[i][begin_coordinates+8] = x*y*y;

	}
      
    }
  
  ifs.close(); 
  return true;
}






/*bool LoadDataFromIndivTrainingConfig(const bool bbase, INT_VCTR_1D& isBase, DBL_VCTR_2D& Xtild, INT_VCTR_3D& X, INT_VCTR_2D& QualVar, INT_VCTR_2D& LevelPopSize, INT_VCTR_1D& isBase_valid, DBL_VCTR_2D& Xtild_valid, INT_VCTR_3D& X_valid, INT_VCTR_2D& QualVar_valid, INT_VCTR_2D& LevelPopSize_valid, const INT_VCTR_1D trainingIndiv, int ER, int EC, const string& File)
{
	
	if (isBase.size() == 0 || isBase_valid.size() == 0)
	{
		cerr << "Uninitialized isBase - LoadDataFromTrainingConfig" << endl;
		exit(EXIT_FAILURE);
	}
	
	if (Xtild.size() == 0 || Xtild_valid.size() == 0)
	{
		cerr << "Uninitialized Xtild - LoadDataFromTrainingConfig" << endl;
		exit(EXIT_FAILURE);
	}

	if (X.size() == 0 || X_valid.size() == 0)
	{
		cerr << "Uninitialized X - LoadDataFromTrainingConfig" << endl;
		exit(EXIT_FAILURE);
	}

	if (Xtild.size() != X.size() || isBase.size() != X.size() || Xtild_valid.size() != X_valid.size() || isBase_valid.size() != X_valid.size())
	{
		cerr << "Mismatched Xtild and X - LoadDataFromTrainingConfig" << endl;
		exit(EXIT_FAILURE);
	}
	if (Xtild.size() != trainingIndiv.size())
	{
		cerr << "Mismatched Xtild and Number of Training Individuls - LoadDataFromTrainingConfig" << endl;
		exit(EXIT_FAILURE);
	}
	int i;
	int itraining=0;  
	int ivalidation=0;
	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cerr << "Cannot open file " << File << " - LoadDataFromTrainingConfig" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(Xtild.size()) + static_cast<int>(Xtild_valid.size()); // Total Number of Individuals
	int NT = static_cast<int>(trainingIndiv.size());

	//cout<<"Xtild.size: "<<M<<"  Xtild[0].size: "<<B<<endl;
	//cout<<"X.size: "<<N<<"  X[0].size: "<<A<<"  X[0][0].size: "<<L<<endl;

	// Load and discard the extra rows.
	char extraRow[10240];
	for (int er = 0; er < ER; er++)
	{
		ifs.getline(extraRow, 10240);
	}
	
	int nextInd=1; // Training Configuration use Ordered Number from 1 to M
	for (int trainInd=0; trainInd < NT; trainInd++)
	{
		for (i = nextInd; i <trainingIndiv[trainInd]; i++)
		{
			//cout << "individual " << i << " in Validation - row " << ivalidation << endl;
			LoadOneIndividualData(bbase, isBase_valid, Xtild_valid, X_valid, QualVar_valid, LevelPopSize_valid, EC, ifs, ivalidation);
			ivalidation++;
		}
		//cout << "individual " << trainingIndiv[trainInd] << " in Training - row " << itraining << endl;
		LoadOneIndividualData(bbase, isBase, Xtild, X, QualVar, LevelPopSize, EC, ifs, itraining);
		itraining++;
		nextInd=trainingIndiv[trainInd]+1;
	}
	for (i=nextInd; i<=M; i++)
	{
		//cout << "individual " << i << " in Validation - row " << ivalidation << endl;
		LoadOneIndividualData(bbase, isBase_valid, Xtild_valid, X_valid, QualVar_valid, LevelPopSize_valid, EC, ifs, ivalidation);
		ivalidation++;
	}

	ifs.close();

	return true;
}

*/

/*void LoadOneIndividualData(const bool bbase, INT_VCTR_1D& isBase, DBL_VCTR_2D& Xtild, INT_VCTR_3D& X, INT_VCTR_2D& QualVar, INT_VCTR_2D& LevelPopSize, int EC,
ifstream &ifs, int i)
{
	int a,l,b;
	int begin_quantitative=1;
	int B = static_cast<int>(Xtild[0].size()); // Number of Covariables  + 1 (Intercept)
	int N = static_cast<int>(X.size());
	int A = static_cast<int>(X[0].size());
	int L = static_cast<int>(X[0][0].size());
	int nbQualLevels = static_cast<int>(LevelPopSize.size());

	for (a = 0; a < A; a++)
	{
		// Load and discard the extra columns.
		string discarded;
		for (int ec = 0; ec < EC; ec++)
		{
			ifs >> discarded;
		}
		
		//Base 
		if (bbase) 
		{
			ifs >> isBase[i];
		}
		if (nbQualLevels>0)
		{ 
			begin_quantitative=nbQualLevels;
			ifs >> QualVar[i];
			QualVar[i]--; //user  writes classes from 1 to nbQualLevels but we use labels from 0 to (nbQualLevels-1)
			if (a==0) LevelPopSize[QualVar[i]]++;
			//the column for the level 0 is deleted (cf Saporta p.398)
			//new column(level i!=0) = old column(level i) - old column(level 0)
			if (QualVar[i]==0)//level 0
			{
				for (b=1; b<nbQualLevels;b++)
				{
					Xtild[i][b]=-1;
				}
			}
			else
			{
				for (b=1; b<nbQualLevels;b++)
				{
					if (b!=QualVar[i])
					{
						Xtild[i][b]=0;
					}
					else
					{
						//cout<<b<<" = "<<QualVar[i];
						Xtild[i][b]=1;
					}
				}
			}
		}
		for (b = begin_quantitative; b < B; b++)
		{
			ifs >> Xtild[i][b];
		}
		
		if (i < N)
		{
			for (l = 0; l < L; l++)
			{
				ifs >> X[i][a][l];
			}
		}
		else
		{
			for (l = 0; l < L; l++)
			{
				ifs >> discarded;
			}
		}
	}
}
*/

/*bool LoadDataFromLociTrainingConfig(const bool bbase, INT_VCTR_1D& isBase, DBL_VCTR_2D& Xtild, INT_VCTR_3D& X, INT_VCTR_3D& Xval, INT_VCTR_2D& QualVar, INT_VCTR_2D& LevelPopSize, const INT_VCTR_1D trainingLoci, int ER, int EC, const string& File)
{
	
	if (isBase.size() == 0)
	{
		cerr << "Uninitialized isBase - LoadData" << endl;
		exit(EXIT_FAILURE);
	}
	
	if (Xtild.size() == 0)
	{
		cerr << "Uninitialized Xtild - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (X.size() == 0)
	{
		cerr << "Uninitialized X - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (Xval.size() == 0)
	{
		cerr << "Uninitialized Xval - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (Xtild.size() < X.size() || isBase.size() < X.size())
	{
		cerr << "Mismatched Xtild and X - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int i, a, l, b, ltraining, lvalidation, nextLocus, trainLocus, locus;
	int begin_quantitative=1;

	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cerr << "Cannot open file " << File << " - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(Xtild.size());
	int B = static_cast<int>(Xtild[0].size());
	int N = static_cast<int>(X.size());
	int Nval = static_cast<int>(Xval.size());
	int A = static_cast<int>(X[0].size());
	int Aval = static_cast<int>(Xval[0].size());
	int L = static_cast<int>(X[0][0].size());
	int Lval = static_cast<int>(Xval[0][0].size());
	int nbQualLevels = static_cast<int>(LevelPopSize.size());

	if ( (N!=Nval) || (A!=Aval) )
	{
		cerr << "Mismatched X and Xval - LoadData" << endl;
		cerr << "N!=Nval or A!=Aval" << endl;
		exit(EXIT_FAILURE);
	}

	// Load and discard the extra rows.
	char extraRow[10240];
	for (int er = 0; er < ER; er++)
	{
		ifs.getline(extraRow, 10240);
	}
	
	for (i = 0; i < M; i++)
	{
		for (a = 0; a < A; a++)
		{
			// Load and discard the extra columns.
			string discarded;
			for (int ec = 0; ec < EC; ec++)
			{
				ifs >> discarded;
			}
			
			//Base 
			if (bbase) 
			{
				ifs >> isBase[i];
			}
			if (nbQualLevels>0)
			{ 
				begin_quantitative=nbQualLevels;
				ifs >> QualVar[i];
				QualVar[i]--; //user  writes classes from 1 to nbQualLevels but we use labels from 0 to (nbQualLevels-1)
				if (a==0) LevelPopSize[QualVar[i]]++; 
				//the column for the level 0 is deleted (cf Saporta p.398)
				//new column(level i!=0) = old column(level i) - old column(level 0)
				if (QualVar[i]==0)//level 0
				{
					for (b=1; b<nbQualLevels;b++)
					{
						Xtild[i][b]=-1;
					}
				}
				else
				{
					for (b=1; b<nbQualLevels; b++)
					{
						if (b!=QualVar[i])
						{
							Xtild[i][b]=0;
						}
						else
						{
							//cout<<b<<" = "<<QualVar[i];
							Xtild[i][b]=1;
						}
					}
				}
			}
			for (b = begin_quantitative; b < B; b++)
			{
				ifs >> Xtild[i][b];
			}
			
			if (i < N)
			{

				nextLocus=1; // Training Configuration use Ordered Number from 1 to LT
				ltraining=0;
				lvalidation=0;
				for (trainLocus=0; trainLocus < L; trainLocus++)
				{
					//if (i==1) cout << "trainLocus:" << trainLocus << "  nextLocus:" << nextLocus << "  trainingLoci[trainLocus]:" << trainingLoci[trainLocus] << endl;
					for (locus = nextLocus; locus <trainingLoci[trainLocus]; locus++)
					{
						//  if (i==1) cout << "Locus " << locus << " in Validation - col " << lvalidation << endl;
						ifs >> Xval[i][a][lvalidation];
						lvalidation++;    
					}
					nextLocus=trainingLoci[trainLocus]+1;
					//if (i==1) cout << "Locus " << trainingLoci[trainLocus] << " in Training - col " << ltraining << endl;
					ifs >> X[i][a][ltraining];
					ltraining++;
				}
				for (locus=nextLocus; locus<=(L+Lval); locus++)
				{
					//if (i==1) cout << "Locus " << locus << " in Validation - col " << lvalidation << endl;
					ifs >> Xval[i][a][lvalidation];
					lvalidation++;
				}
				
			}
			else
			{
				for (l = 0; l < (L+Lval); l++)
				{
					ifs >> discarded;
				}

			}
		}
	}
	ifs.close();

	return true;
}*/














