#include <chartdir.h>
#include <iostream>
#include <cstdlib>

#include <set>
#include <sstream>

#include "PlotUtilities.h"
//#include "VoronoiGeometry.h"

using namespace std;

#define NUM_COL 32
#define NUM_SYM 32

int COLORS[NUM_COL] = {
	0xFF3333, 0x33FF33, 0x6666FF, 0xFFFF00, 0xFF66FF, 0x99FFFF, 0xFFCC33, 0xCCCCCC, 
	0xCC9999, 0x339966, 0x999900, 0xCC3300, 0x669999, 0x993333, 0x006600, 0x990099, 
	0xFF9966, 0x99FF99, 0x9999FF, 0xCC6600, 0x33CC33, 0xCC99FF, 0xFF6666, 0x99CC66, 
	0x009999, 0xCC3333, 0x9933FF, 0xFF0000, 0x0000FF, 0x00FF00, 0xFFCC99, 0x999999 
};

// Draw a bar chart to show the assignment probabilities.
int PlotAP(const char* FileName, const DBL_VCTR_2D& ap, const char* title)
{
	int i, k;

	int N = static_cast<int>(ap.size());
	int K = static_cast<int>(ap[0].size());

	DBL_VCTR_2D chartData;
	InitVctr2D(chartData, K, N, 0.0);
	for (i = 0; i < N; i++)
	{
		for (k = 0; k < K; k++)
		{
			chartData[k][i] = ap[i][k];
		}
	}

	DBL_VCTR_1D chartLabels(N, 0.0);
	for (i = 0; i < N; i++)
	{
		// Make sure the first index is 1 not 0.
		chartLabels[i] = i + 1;
	}

	int barWidth = 30;
	XYChart* c = new XYChart(barWidth * N + 100, 600, 0xFFFFCC, 0, 1);
	c->setPlotArea(50, 50, barWidth * N, 500, 0xFFFFFF);
	c->setClipping();
	c->addTitle(title, "timesbi.ttf", 14, 0xFFFFFF
		)->setBackground(0x804020);
	c->xAxis()->setLabels(DoubleArray(&chartLabels[0], N), "{value|0}");
	BarLayer *layer = c->addBarLayer(Chart::Percentage);
	c->yAxis()->setLabelFormat("{={value}*0.01}");
	c->yAxis()->setTickDensity(50);

	for (k = 0; k < K; k++)
	{
		layer->addDataSet(DoubleArray(&chartData[k][0], N));
	}
	layer->setBorderColor(Chart::Transparent);
	bool bMade = c->makeChart(FileName);
	if (!bMade)
	{
		cerr << "Unable to make chart - PlotAP" << endl;
	}
	delete c;

	return 0;
}


// Draw a bar chart to show the assignment probabilities predicted for each class.
int PlotPPMqual(const char* FileName, const DBL_VCTR_3D& ap)
{

  cout << "enter plotPPMqual" << endl;
  int i,k,var;
  
  int nQualVar = static_cast<int>(ap.size());
  int K  = static_cast<int>(ap[0][0].size());
  int N;  // N = number of classes for the current qualitative variable

  XYChart* c[nQualVar];
  
  int step = 50;
  int firstStep = 0;
  int barWidth = 30;
  int Y = 600 + 2*step;
  int X =  step*(nQualVar+1) + firstStep; 
  // Position for charts on mc:
  int y = step;
  int x = firstStep + step;
  for (var=0; var<nQualVar; var++)
    {
      N  = static_cast<int>(ap[var].size());
      X += barWidth * N + 100;
      c[var] = new XYChart(barWidth * N +100, Y-2*step , 0xFFFFCC, 0, 0);
    }

  MultiChart* mc = new MultiChart(X, Y, 0xFFFFCC, 0, 1);
  mc->addTitle("Predicted Assignement Probabilities", "timesbi.ttf", 14, 0xFFFFFF)->setBackground(0x804020);

  
  for (var=0; var<nQualVar; var++)
    {
      N  = static_cast<int>(ap[var].size()); // N = nLevels
	    
      DBL_VCTR_2D chartData;
      InitVctr2D(chartData, K, N, 0.0);
      for (i = 0; i < N; i++)
	{
	  for (k = 0; k < K; k++)
	    {
	      chartData[k][i] = ap[var][i][k];
	    }
	}
	    
      DBL_VCTR_1D chartLabels(N, 0.0);
      for (i = 0; i < N; i++)
	{
	  // Make sure the first index is 1 not 0.
	  chartLabels[i] = i + 1;
	}

	    
      c[var]->setPlotArea(50, 50, barWidth * N, 500, 0xFFFFFF);
      c[var]->setClipping();
      stringstream title;
      title << "Qualitative variable n" << var+1;
      c[var]->addTitle(title.str().c_str(), "timesbi.ttf", 10);//)->setBackground(0x804020);
      c[var]->xAxis()->setLabels(DoubleArray(&chartLabels[0], N), "{value|0}");
      BarLayer *layer = c[var]->addBarLayer(Chart::Percentage);
      c[var]->yAxis()->setLabelFormat("{={value}*0.01}");
      c[var]->yAxis()->setTickDensity(50);

      for (k = 0; k < K; k++)
	{
	  layer->addDataSet(DoubleArray(&chartData[k][0], N));
	}
      layer->setBorderColor(Chart::Transparent);
	    
      mc->addChart(x,y,c[var]);
      x += barWidth*N+100 + step;

    }


  bool bMade = mc->makeChart(FileName);
  if (!bMade)
    {
      cerr << "Unable to make chart - PlotPPMqual" << endl;
    }
  else
    {
      cout << "Chart made - PlotPPMqual" << endl;
    }
	
  for (i = 0; i < nQualVar; i ++)
    {
      delete c[i];
    }
	
  delete mc;
	
  return 0;
}

// Draw a line chart to show the log-likelihood history.
int PlotLH(const char* FileName, const DBL_VCTR_1D& lh)
{
	int i;

	DBL_VCTR_1D lhCopy = lh;
	int NoDP = static_cast<int>(lhCopy.size());  // Number of Data Points

	const double NEG_INF = -1.0E+300;
	for (i = 0; i < NoDP; i++)
	{
		if (lhCopy[i] < NEG_INF)
		{
			cerr << "Data Out Of Range, No Chart Plotted - PlotLH" << endl;
			return -1;
		}
	}

	DBL_VCTR_1D chartLabels(NoDP, 0.0);
	for (i = 0; i < NoDP; i++)
	{
		// Make sure the first index is 1 not 0.
		chartLabels[i] = i + 1;
	}

	XYChart* c = new XYChart(900, 600, 0xFFFFCC, 0, 1);
	c->setPlotArea(50, 50, 800, 500, 0xFFFFFF);
	c->setClipping();
	c->addTitle("Log-Likelihood History", "timesbi.ttf", 14, 0xFFFFFF
		)->setBackground(0x804020);
	c->addLineLayer(DoubleArray(&lhCopy[0], NoDP));
	c->xAxis()->setLabels(DoubleArray(&chartLabels[0], NoDP), "{value|0}");
	int nStep = NoDP / 10;
	if (nStep > 0)
	{
		c->xAxis()->setLabelStep(nStep);
	}
	bool bMade = c->makeChart(FileName);
	if (!bMade)
	{
		cerr << "Unable to make chart - PlotLH" << endl;
	}
	delete c;

	return 0;
}


//Draw the history of beta convergence
// Warning betaHist(K-1,B,TotalSweep) (ie Beta History is not stored as in TessAdmCov)
int PlotBetaHist(const char* FileName, const DBL_VCTR_3D& betaHist, const INT_VCTR_1D& nQualLevels, const int nbQuantVar, const int trendDegree)
{
	int i,k,b,var,level;
	string svar,slevel;

	if (betaHist.size() == 0)
	{
		cerr << "Uninitialized betaHist - PlotBetaHist" << endl;
		exit(EXIT_FAILURE);
	}
	
	int K = static_cast<int>(betaHist.size());
	int B = static_cast<int>(betaHist[0].size());
	int NoDP = static_cast<int>(betaHist[0][0].size());
	const double NEG_INF = -1.0E+300;
	
	int nQualVar = static_cast<int>(nQualLevels.size());
	
	int cptChart = 0;
	XYChart* c[K*B];
	
	int xSize = 300;
	int ySize = 200;
	int step = 50;
	int firstStep = 50;
	
	int X = xSize*B + step*(B+1) + firstStep; 
	int Y = ySize*K + step*(K+1) + firstStep;	
	MultiChart* mc = new MultiChart(X, Y, 0xFFFFCC, 0, 1);
	mc->addTitle("Regression Coefficients", "timesbi.ttf", 14, 0xFFFFFF)->setBackground(0x804020);
	
	int yTextPos = step + firstStep + ySize/2;
	int x = 0;
	int y = 0;
	
	for (k = 0; k < K; k++)
	{
		stringstream clust;
		clust << "Cluster " << k + 1;
		mc->addText((step+firstStep)/2,yTextPos,clust.str().c_str(),"timesbi.ttf",12,0x000000,5,90);
		yTextPos += ySize + step;
		for (b = 0; b < B; b ++)
		{
			DBL_VCTR_1D bh;
			InitVctr1D(bh, NoDP, 0.0);
			for (i = 0; i < NoDP; i ++)
			{
				bh[i] = betaHist[k][b][i];
			}
			
			for (i = 0; i < NoDP; i++)
			{
				if (bh[i] < NEG_INF)
				{
					cerr << "Data Out Of Range, No Chart Plotted - PlotBetaHist" << endl;
					return -1;
				}
			}
			
			DBL_VCTR_1D chartLabels(NoDP, 0.0);
			for (i = 0; i < NoDP; i++)
			{
				// Make sure the first index is 1 not 0.
				chartLabels[i] = i + 1;
			}
			
			
// 			XYChart* c = new XYChart(xSize, ySize, 0xFFFFCC, 0, 0);
			c[cptChart] = new XYChart(xSize, ySize, 0xFFFFCC, 0, 0);
			c[cptChart]->setPlotArea(20, 20, 260, 160, 0xFFFFFF);
			c[cptChart]->setClipping();
// 			stringstream title;
// 			title << "Cluster " << k+1 << " Predictor " << b+1;
// 			c[cptChart]->addTitle(title.str().c_str(), "timesbi.ttf", 10, 0xFFFFFF)->setBackground(0x804020);
			c[cptChart]->addLineLayer(DoubleArray(&bh[0], NoDP));
			c[cptChart]->xAxis()->setLabelStyle(0,6);
			c[cptChart]->yAxis()->setLabelStyle(0,6);
			c[cptChart]->xAxis()->setLabels(DoubleArray(&chartLabels[0], NoDP), "{value|0}");
			int nStep = NoDP / 10;
			if (nStep > 0)
			{
				c[cptChart]->xAxis()->setLabelStep(nStep);
			}
						
			x = (1+b)*step + b*xSize + firstStep;
			y = (1+k)*step + k*ySize + firstStep;
			
			mc->addChart(x,y,c[cptChart]);
			
			cptChart ++;

			
		}
	}
	
	
	string pred = "Intercept";
	int xOffset = 10*pred.size();
				
	int xTextPos = step + firstStep + xSize/2 -xOffset/2;
	int align = 7;
	mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
	xTextPos += xSize + step + xOffset/2;
	
	for (var = 0; var < nQualVar; var++)
	{
		stringstream out;
		out << var+1;
		svar = out.str();
		for (level = 0; level < nQualLevels[var]-1; level++)
		{
			string slev;
			stringstream outlev;
			// for user the level n1 is the one we transform in the "-1 -1 -1 .." line
			// so what we begin to display is the level n2
			outlev << level+2; 
			slev = outlev.str();	
			pred = "Qual. Var "+svar+" level"+slev;
			xOffset = 10*pred.size();
			xTextPos -= xOffset/2;
			mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
			xTextPos += xSize + step + xOffset/2;		
		}
	}
	
	for (var = 0; var < nbQuantVar; var++)
	{
		stringstream out;
		out << var+1;
		svar = out.str();
		pred = "Quant. Var "+svar;
		xOffset = 10*pred.size();
		xTextPos -= xOffset/2;
		mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
		xTextPos += xSize + step + xOffset/2;		
	}

	if (trendDegree > 0)
	  {
	        pred = "Longitude";
		xOffset = 10*pred.size();
		xTextPos -= xOffset/2;
		mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
		xTextPos += xSize + step + xOffset/2;
		
		pred = "Latitude";
		xOffset = 10*pred.size();
		xTextPos -= xOffset/2;
		mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
		xTextPos += xSize + step + xOffset/2;
		if (trendDegree > 1)
		{
			pred = "Longitude^2";
			xOffset = 10*pred.size();
			xTextPos -= xOffset/2;
			mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
			xTextPos += xSize + step + xOffset/2;
			
			pred = "Latitude^2";
			xOffset = 10*pred.size();
			xTextPos -= xOffset/2;
			mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
			xTextPos += xSize + step + xOffset/2;
			
			pred = "Longitude * Latitude";
			xOffset = 10*pred.size();
			xTextPos -= xOffset/2;
			mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
			xTextPos += xSize + step + xOffset/2;
			if (trendDegree > 2)
			{
				pred = "Longitude^3";
				xOffset = 10*pred.size();
				xTextPos -= xOffset/2;
				mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
				xTextPos += xSize + step + xOffset/2;
				
				pred = "Latitude^3";
				xOffset = 10*pred.size();
				xTextPos -= xOffset/2;
				mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
				xTextPos += xSize + step + xOffset/2;
				
				pred = "Longitude^2 * Latitude";
				xOffset = 10*pred.size();
				xTextPos -= xOffset/2;
				mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
				xTextPos += xSize + step + xOffset/2;
				
				pred = "Longitude * Latitude^2";
				xOffset = 10*pred.size();
				xTextPos -= xOffset/2;
				mc->addText(xTextPos,(step+firstStep)/2,pred.c_str(),"timesbi.ttf",12,align);
			}
		}
	}
	
	bool bMade = mc->makeChart(FileName);
	if (!bMade)
	{
		cerr << "Unable to make chart - PlotBetaHist" << endl;
	}
	
	for (i = 0; i < K*B; i ++)
	{
		delete c[i];
	}

	delete mc;

	return 0;
	
}
