#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <chartdir.h>
#include <set>
#include <string.h>

#include "gsl/gsl_randist.h"
#include <gsl/gsl_rng.h>


#include "PredictUtilities.h"
#include "IOUtilities.h"
#include "PlotUtilities.h"
#include "RNGUtilities.h"



void  ShowUsagePrediction()
{
  cout << endl;
  cout << endl;
  cout << "*** Prediction Utilities ***" << endl;
  cout << endl;
  cout << "Typical Usages:" << endl;
  cout << "     * No Qualitative Covariates in the Data:\n     pops -predict -BETAbeta -newFnewF -newNnewN -newCnewC -newRnewR -XX -XLXL -TT -admadm -oo" << endl;
  cout << endl;
  cout << "     * Qualitative Covariates in the Data:\n     pops -predict -BETAbeta -newFnewF -newNnewN -newNnewN -newCnewC -XX -XLXL -TT -admadm -oo -FF -NN -AA -LL -cc -rr" << endl;
  cout << endl;
  cout << endl;
  cout << "Parameters:" << endl;
  cout << "     * Parameters for Data without Qualitative Covariates:" << endl;
  cout << "-newFnewF: newF = File Name of Data File containing the New Values of Covariates" << endl;
  cout << "                  If file name contains spaces, input it as \"F\"." << endl;
  cout << "-newNnewN: N = Number of Individuals in newF" << endl;
  cout << "-newNnewC: C = Number of Extra Columns in newF" << endl;
  cout << "-newNnewR: R = Number of Extra Rows in newF" << endl;
  cout << "-oo:       o = Name for Outputs " <<endl;

  cout << endl;
  cout << endl;

  cout << "-BETAbeta: beta = File Name of Data File containing the Estimated Regression Coefficients from which you want to predict" << endl;
  cout << "                  If file name contains spaces, input it as \"beta\"." << endl;
  cout << endl;
  cout << "   !Warning: Following parameters have to correspond to the parameters of the run that estimated beta!" << endl;
  cout << "-admadm: adm = Was this run with or without admixture?" << endl;
  cout << "         adm = y, Yes" << endl;
  cout << "         adm = n, No" << endl;
  cout << "-KK:     K = Number of Clusters" << endl;
  cout << "-XLXL:   XL = Number of Qualitative Covariates" << endl;
  cout << "-XX:     X = Number of Quantitative Covariates (other than spatial coordinates)" << endl;
  cout << "-TT:     T = Degree of Trend" << endl;
  cout << "             -1: NO Spatial Coord. in datafile F" << endl; 
  cout << "              0: Spatial Coord. NOT used" << endl;
  cout << "             {1,2,3}: Degree 1, 2 or 3" << endl;
  cout << endl;

  cout << endl;
  cout << endl;
  cout << "     * Additional Parameters for Data with Qualitative Covariates:" << endl;
  cout << "POPS need additional informations to load the original data (with genotypes) that were used for the estimation of beta" << endl;
  cout << "Values for these parameters should be the same as when you launched pops to analyse the original data" << endl;
  cout << "-FF: F = File Name of Input Data File" << endl;
  cout << "         If file name contains spaces, input it as \"F\"." << endl;
  cout << "-NN: N = Number of Individuals" << endl;
  cout << "-AA: A = Ploidy (1 = Haploid, 2 = Diploid, ...)" << endl;
  cout << "-LL: L = Number of Loci" << endl;
  cout << " Optional:" << endl;
  cout << "-rr:   r = Number of Extra Rows in Data File, Default: 0" << endl;
  cout << "-cc:   c = Number of Extra Columns in Data File, Default: 0" << endl;
  cout << "-spsp: sp = SPecial data format: one individual = one row" << endl;
  cout << "       sp = y, Yes" << endl;
  cout << "       sp = n, No (Default)" << endl;
  cout << "-ref:  The 1st non-extra column of datafile F indicates Individuals to assigned to the Reference Cluster" << endl;
  cout << "       (value 1 for ref. indiv., 0 for others)" << endl;
  
  return;
}


void predictionFromCommandLine(int argc, char* argv[]) {

	string betaF;    	        // Name of the File containing the Estimated Regression Coefficients
	string newF;	        	// Name of the File containing New Data
	int newN;                       // Number of Individuals in New File
	int newER=0;                      // Extra Rows in New file
	int newEC=0;                      // Extra Columns in New file
 	bool admixture;                 // Admixture model ?
	int K;			        // Number of Cluster
    	int trendDegree = -1;           //Degree of trend (default, no spatial coordinates)
	int nbQuantVar=0;               // Number of Quantitative Covariates other than Spatial Coordinates
	int nbQualVar=0;		// Number of Qualitative Covariates
	string outfile;                 // Name for Prediction Outputs

	// Optional Parameters
	
	bool bplot=true;                     // Should Prediction be display on a Graphical Barplot ?

	// Optional Parameters that HAVE to be given if there are Qualitative Variables in the Data
    
	string F;	        	// Name of the Original Complete Data File (the one with genotypes)
	int N;				// Number of Samples/Individuals in Training data
	int L;				// Number of Loci for Training Data
	int A;				// Haploid, Diploid, ...
	int ER = 0;			// Number of Extra Rows
	int EC = 0;			// Number of Extra Columns
	bool isRecAllele = false;       // Is there recessive alleles in the data set ? For compatibility with tessAdmCov, not used in this version
	bool specialFormat = false;     // Is the data in special format? (one line per individual, even diploid). Then diploid individual: one locus = 2 columns
	bool hasReferenceCol = false;

	string FoI;

	// Boolean
	bool bbetaF = false;
	bool bnewF = false;
	bool bnewN = false;
	bool bnewER = false;
	bool bnewEC = false;
	bool bX = false;
	bool bXL = false;
	bool bT = false;
	bool bK = false;
	bool bAdm = false;
	bool bO = false;

	bool bF = false;
	bool bN = false;
	bool bA = false;
	bool bL = false;


	bool bbase=false;





	// Parse the command line parameters.
	for(int ac = 1; ac < argc; ac++)
	  {
	    switch (argv[ac][1])
	      {
				
	      case 'X':
		if (argv[ac][2]=='L')
		  {
		    nbQualVar = atoi((const char*) &argv[ac][3]);
		    bXL = true;
		  }
		else
		  {
		    nbQuantVar = atoi((const char*) &argv[ac][2]);
		    bX = true;
		  }
		break;
				
	      case 'F':
		F = (const char*) &argv[ac][2];
		bF = true;
		break;

	      case 'N':
		N = atoi((const char*) &argv[ac][2]);
		bN = true;
		break;
				
	      case 'A':
		A = atoi((const char*) &argv[ac][2]);
		bA = true;
		break;

	      case 'L':
		L = atoi((const char*) &argv[ac][2]);
		bL = true;
		break;

	      case 'K':
		K = atoi((const char*) &argv[ac][2]);
		bK = true;
		break;

   	      case 'T':
		trendDegree = atoi((const char*) &argv[ac][2]);
		bT = true;
		break;
 
	      case 'O':
		outfile = (const char*) &argv[ac][2];
		bO = true;
		break;
	      case 'o':
		outfile = (const char*) &argv[ac][2];
		bO = true;
		break;

	      case 'B':
		if ( (argv[ac][2] == 'E') && (argv[ac][3] == 'T')  && (argv[ac][4] == 'A'))
		  {
		    betaF =  (const char*) &argv[ac][5];
		    bbetaF = true;
		  }
		else 
		  {
		    cout << "Unknown Option : " << argv[ac] << endl;
		    ShowUsagePrediction();
		    exit(EXIT_FAILURE);
		  }
		break;
		
	      case 'n':
		if ( (argv[ac][2] == 'e') && (argv[ac][3] == 'w') )
		  {
		    switch (argv[ac][4])
		      {
		      case 'R':
			newER = atoi((const char*) &argv[ac][5]);
			bnewER = true;
			break;

		      case 'C':
			newEC = atoi((const char*) &argv[ac][5]);
			bnewEC = true;
			break;

		      case 'N':
			newN = atoi((const char*) &argv[ac][5]);
			bnewN = true;
			break;

		      case 'F':
			newF = (const char*) &argv[ac][5];
			bnewF = true;
			break;
		      }
		  }
		else 
		  {
		    cout << "Unknown Option - " << argv[ac] << endl;
		    ShowUsagePrediction();
		    exit(EXIT_FAILURE);
		  }
		break;

	      case 'a':
		if ( (argv[ac][2] == 'd') && (argv[ac][3] == 'm') )
		  {
		    if  (argv[ac][4] == 'y') 
		      {
			admixture = true;
			bAdm = true;
		      } 
		    else  if  (argv[ac][4] == 'n') 
		      {
			admixture = false;
			bAdm = true;
		      }
		    else
		      {
			cout << "Unknown Option - " << argv[ac] << endl;
			ShowUsagePrediction();
			exit(EXIT_FAILURE);
		      }
		  }
		else
		  {
		    cout << "Unknown Option - " << argv[ac] << endl;
		    ShowUsagePrediction();
		    exit(EXIT_FAILURE);
		  }
		break;

	      case 'r':
		
		if (argv[ac][2] == 'e')
		  // option -ref
		  {
		    if (argv[ac][3] == 'f')
		      {
			hasReferenceCol=true;
			
		      }
		    else
		      {
			cout << "Unknown Option - " << argv[ac] << endl;
			ShowUsagePrediction();
			exit(EXIT_FAILURE);
		      }
		  }
		else if (argv[ac][2] == 'c')
		  {
		    // recessive alleles ?
		    if (argv[ac][3] == 'y')
		      {
			isRecAllele = true;
		      }
		    if (argv[ac][3] == 'n')
		      {
			isRecAllele = false;
		      }
		    if (argv[ac][3] != 'y' && argv[ac][3] != 'n')
		      {
			cout << "Unknown Option - " << argv[ac] << endl;
			ShowUsagePrediction();
			exit(EXIT_FAILURE);
		      }	
		  }
		else
		  {
		    ER = atoi((const char*) &argv[ac][2]);
		  }
		break;

	      case 'c':
		EC = atoi((const char*) &argv[ac][2]);
		break;
		
	      case 'i':
		FoI = (const char*) &argv[ac][2];
		break;

	     
	      case 's':
		// Special data format
		if (argv[ac][2] == 'p')
		  {
		    if (argv[ac][3] == 'y')
		      {
			specialFormat = true;
		      }
		    else if (argv[ac][3] == 'n')
		      {
			specialFormat = false;
		      }
		    else
		      {
			cout << "Unknown Option - " << argv[ac] << endl;
			ShowUsagePrediction();
			exit(EXIT_FAILURE);
		      }
		  }
		else
		  {
		    cout << "Unknown Option - " << argv[ac] << endl;
		    ShowUsagePrediction();
		    exit(EXIT_FAILURE);
		  }
		break;

	      case 'p':
		if ( (argv[ac][2] == 'l') && (argv[ac][3] == 'o')&& (argv[ac][4] == 't') )
		  {
		    if (argv[ac][5] == 'y')
		      {
			bplot = true;
			break;
		      }
		    else if (argv[ac][5] == 'n')
		      {
			bplot = false;
			break;
		      }
		    else
		      {
			cout << "Unknown Option - " << argv[ac] << endl;
			ShowUsagePrediction();
			exit(EXIT_FAILURE);
		      }
		    
		  }
		else if (strcmp(argv[ac],"-predict")==0)
		  {
		    break;		    
		  }
		else
		  {
		    cout << "Unknown Option - " << argv[ac] << endl;
		    ShowUsagePrediction();
		    exit(EXIT_FAILURE);
		  }
		break;

	     
	 	
	      default:  // Unknown parameters.
		cout << "Unknown Option - " << argv[ac] << endl;
		ShowUsagePrediction();
		exit(EXIT_FAILURE);
	      }
	  }
	


if (!bbetaF || !bnewF || !bnewN || !bK ||  !bT || !bX || !bXL || !bAdm || !bO)
  {
    string missingopt="";
    if (!bbetaF) missingopt+=" -BETA" ;
    if (!bnewF) missingopt+=" -newF" ;
    if (!bnewN) missingopt+=" -newN" ;
    if (!bK) missingopt+=" -K" ;
    if (!bT) missingopt+=" -T" ;
    if (!bX) missingopt+=" -X" ;
    if (!bXL) missingopt+=" -XL" ;
    if (!bAdm) missingopt+=" -adm" ;
    if (!bO) missingopt+=" -o (or -O) " ;

    cout << "\n Mandatory arguments are missing, please check details for: " << missingopt << " with pops -predict | more " << endl;
    exit(EXIT_FAILURE);
  }




if (trendDegree < -1 || trendDegree > 3)
  {
    cout << "The trend degree must be in {-1,0,1,2,3}." << endl;
    cout << "(value -1 if the datafile do NOT contained spatial coordinates)" << endl;
    exit(EXIT_FAILURE);
  }

bool hasSpatialCoord = (trendDegree!=-1)? true:false;
 
 const gsl_rng_type *T;
 gsl_rng_env_setup();
 T = gsl_rng_default;
 gsl_rng* rng = NULL;
 AllocRNG(rng, T);



vector< set< int > > levelS;
int nbQualColumns = 0;
 if (nbQualVar>0) 
   {
     
     if (!bF || !bN || !bA || !bL )
       {
	 cout << "Error: With qualitative covariates, more parameters are required to compute prediction, see details for -F -N -A and -L" << endl;
	 ShowUsagePrediction();
	 exit(EXIT_FAILURE);
       }
  

     bool loadedQual =LoadQualInfoFromData(N,nbQualVar,nbQuantVar,L,A, ER, EC,specialFormat,hasReferenceCol,isRecAllele,hasSpatialCoord, levelS, F);
     if (!loadedQual || ((int)levelS.size() != nbQualVar) )
       {
	 cerr << "Error while loading info. Please verify command line options." << endl;
	 cerr << "Have you changed the main datafile since the project creation ? If yes, the project parameters may be obsolete. Please try to create a new project or use the same data as before." << endl;
	 ShowUsagePrediction();
	 cout << "\n Error while loading info. Please verify command line options." << endl;
	 cout << "Have you changed the main datafile since the project creation ? If yes, the project parameters may be obsolete. Please try to create a new project or use the same data as before." << endl;
	 exit(EXIT_FAILURE);
       }
     for (int var = 0; var < nbQualVar; var++)
       {
	 nbQualColumns +=  static_cast<int> (levelS[var].size()) - 1;
       }
   }


 DBL_VCTR_2D newDataXtild;
  // if there are coordinates in this project, we compute the trend of maximal degree (ie degree 3 -> 9 columns in Xtild)
  int nbXtildCol = 1 + nbQualColumns + nbQuantVar + (hasSpatialCoord ? 9:0);
  InitVctr2D(newDataXtild,newN,nbXtildCol,0.0);
  DBL_VCTR_2D newDataC;
  InitVctr2D(newDataC,newN,2,0.0);
  

  bool loaded = LoadNewData(newDataC, newDataXtild, newER, newEC, nbQuantVar, hasSpatialCoord, levelS, newF);

  if (!loaded)
    {
      cerr << "Problem while loading data. Please verify command line options" << endl;
      ShowUsagePrediction();
      cout << "Problem while loading data. Please verify command line options" << endl;
      exit(EXIT_FAILURE);
    }
  
  bool predictDone = predict(admixture, betaF, newDataXtild, K, trendDegree, outfile, bplot, rng);
  
  if (!predictDone)
    {
      cerr << "Problem while prediction computation. Please verify command line options" << endl;
      ShowUsagePrediction();
      cout << "Problem while prediction computation. Please verify command line options" << endl;
      exit(EXIT_FAILURE);
    }

return;
  
}

bool predict(const bool adm, const string& betaFile, DBL_VCTR_2D& Xtild, const int K, const int trendDegree, const string& outFile, const bool plotPredBarplot, gsl_rng*& rng)
{

  int N = static_cast<int> (Xtild.size());
  int Bmax = static_cast<int> (Xtild[0].size());
  int B;
  bool bPredict=false;
  switch (trendDegree) 
    {
    case -1:
    // no spatial coord
      B = Bmax;
      break;
    default:
      // We kept the B first columns depending on the trend degree of the current run
      // Ex: the 4 last columns of Xtild correspond to the trend terms of degree 3
      B = Bmax - 9 + (trendDegree +1)*(trendDegree +2)/2 - 1; 
  }

  DBL_VCTR_2D nClusterIndiv;
  InitVctr2D(nClusterIndiv,N,K,0.0);

   if (adm) 
    {
      bPredict=predictAdm(betaFile, Xtild, K, B, nClusterIndiv, rng);

    }
  else
    {
      bPredict=predictNoAdm(betaFile, Xtild, K, B, nClusterIndiv, rng);
   
    }
  

   SaveToMyFileClumppFormat(nClusterIndiv,outFile + ".txt",true,"Predicted Clustering Probabilities for Samples");

   if (plotPredBarplot)
     {
       string pngFile = outFile + ".png";
       PlotAP( pngFile.c_str(), nClusterIndiv, "Predicted Assignement Probilities");

     }

   if (bPredict)
     {
       cout << "Predictions saved in to " << outFile+".txt" << endl;
       if (plotPredBarplot)
	 {
	   cout << " and to " << outFile+".png" << endl;
	 }
     }
  

    return bPredict;

}

bool predictAdm(const string& betaFile, DBL_VCTR_2D& Xtild, const int K, const int B,  DBL_VCTR_2D& nClusterIndiv, gsl_rng*& rng)
{
  int N = static_cast<int> (Xtild.size());

  DBL_VCTR_2D Pred, beta;
  InitVctr2D(Pred, N, K, 0.0);
  InitVctr2D(beta, B, K, 0.0);
  LoadBeta(beta, betaFile);


  TroncateAndMult(Pred,Xtild,beta); //  in R software: Xtild[,1:B] %*% beta

  //SaveToMyFile(beta,"betaAdm.txt");
  //SaveToMyFile(Pred,"predAdm.txt");


  DBL_VCTR_2D QPredict;
  InitVctr2D(QPredict,N,K,0.0);
  DBL_VCTR_1D dpqPredict;
  InitVctr1D(dpqPredict,K,0.0);

  int i,k;
  //for (predict = 0; predict < nb_predict; predict++)
  // {
      for (i = 0; i < N; i++)
	{
	  for (k = 0; k < K; k++)
	    {
	      dpqPredict[k] = exp(Pred[i][k]); // + gsl_ran_gaussian(rng,sqrt(spSweep.sigma[k])));
	    }
	  gsl_ran_dirichlet(rng, K, &dpqPredict[0], &QPredict[i][0]);

	  nClusterIndiv=QPredict;
	  
	  //Now QPredict[i,] is fully computed. We predict every loci' population based on Qik
	  /*for (a = 0; a < A; a ++)
	    {
	      for (l = 0; l < L; l ++)
		{
		  int zial = SelectANum(rng, QPredict[i]);
		  nb_cluster_loci[i][a][l][zial]++;
		  nClusterIndiv[i][zial]++;
		 
		}
		}*/
	}
      //}


  return true;
}

bool predictNoAdm(const string& betaFile, DBL_VCTR_2D& Xtild, const int K, const int B, DBL_VCTR_2D& nClusterIndiv, gsl_rng*& rng)
{
  int N = static_cast<int> (Xtild.size());

  DBL_VCTR_2D Pred, beta;
  InitVctr2D(Pred, N, K-1, 0.0);
  InitVctr2D(beta, B, K-1, 0.0);
  LoadBeta(beta, betaFile);

  TroncateAndMult(Pred,Xtild,beta); //  in R software: Xtild[,1:nrow(beta)] %*% beta

  //SaveToMyFile(beta,"beta.txt");
  //SaveToMyFile(Pred,"pred.txt");

  double max,tmpPred;
  int zi,i,k;
  int nb_predict = 10;

  for (int predict=0; predict <nb_predict; predict++)
  {
      //calcul pij
      for (i = 0; i < N; i++)
	{
	  max = 0;
	  zi = K-1; //Z[i] = base if max<0 
						
	  for (k=0; k < K-1; k++)
	    {
	      tmpPred = Pred[i][k]+gsl_ran_ugaussian(rng);
	      if (tmpPred > max)
		{
		  max = tmpPred;
		  zi = k;
		}
	    }
	  nClusterIndiv[i][zi] += 1.0/(double)nb_predict;
	}
  }
  
  
  return true;

}

void LoadBeta(DBL_VCTR_2D& beta, const string& File)
{

  if (beta.size()==0 || beta[0].size() ==0)
    {
      
      cerr << "Uninitialized beta - LoadBeta" << endl;
      exit(EXIT_FAILURE);
      
    }
  
  int B = static_cast<int>(beta.size());
  int Kmax = static_cast<int>(beta[0].size()); // can be K (admixture model) or K-1 (no adm. model)
  
  ifstream ifs(File.c_str());
  if (!ifs.good())
    {
      cout << "Cannot open file " << File << " - LoadBeta" << endl;
      exit(EXIT_FAILURE);
      
    }
  
  // in beta file lines correspond to k-values and columns to b-values
  for (int k = 0; k < Kmax; k++)
    {
      for (int b = 0; b < B; b++)
	{
	  ifs >> beta[b][k];  
	}
    }

  return;
}


void TroncateAndMult(DBL_VCTR_2D& Res, const DBL_VCTR_2D& A, const DBL_VCTR_2D& B)

  // A[,1:nrow(B)] %*% B
  // ie use only the B first columns of A for the multiplication
{

  int lig,col,tmp;
 if (A.size()==0 || A[0].size() ==0 || B.size()==0 || B[0].size() ==0)
    {
      
      cerr << " Uninitialized Matrix - TroncateAndMult" << endl;
      exit(EXIT_FAILURE);
      
    }
  
  if (A[0].size() < B.size()){ 
    cerr<<A[0].size()<<" vs "<<B.size()<<endl;
    cerr<<"TroncateAndMult: Matrices have inadequate dimensions to be multiplied, and A cannot be troncated to avoid the problem. Exiting..."<<endl;
    exit(EXIT_FAILURE);
  }
  

  int L=(int)A.size();
  int C=(int)B[0].size();
  int T=(int)B.size();  // ie nrow(B)
  if ((int)Res.size()==0){
    // Res was not initialized
    InitVctr2D(Res,L,C,0.0);
  }else if ( ((int)Res.size()!=L) || ((int)Res[0].size()!=C) ){
    cout<<"TroncateAndMult: Res matrix has inadequate dimensions to contain the result of the matrices multiplication. Exiting..."<<endl;
    exit(EXIT_FAILURE);
  }
  for (lig=0; lig<L; lig++){
    for (col=0; col<C; col++){
      Res[lig][col]=0.0;
      for (tmp=0; tmp<T; tmp++){
	Res[lig][col]+=A[lig][tmp]*B[tmp][col];
      }
    }
  } 
  return; 
}

