#ifndef __STAT_UTILITIES__
#define __STAT_UTILITIES__

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include "VectorUtilities.h"
#include "ranker.h"
#include "ManipVector.h"
#include "MatrixUtilities.h"



/********************************************************
 ***                  STATS UTILITIES                 ***
 ********************************************************/
void MultivariateGauss(DBL_VCTR_1D& mu, DBL_VCTR_2D& v_chol, DBL_VCTR_1D& Y, gsl_rng*& rng);

double TruncGauss(double mu, double sigma, double lower, double upper, gsl_rng*& rng);
double TruncGauss_Left(double mu, double sigma, double lower, gsl_rng*& rng);
double TruncGauss_Right(double mu, double sigma, double upper, gsl_rng*& rng);

void SimulateXtild(DBL_VCTR_2D& Xtild, int N, int beta_length, gsl_rng*& rn);
void MultiProbitModel(INT_VCTR_1D& Z, DBL_VCTR_2D& Xtild, DBL_VCTR_2D& Beta_vrai, int M, int base, gsl_rng*& rng);


//! return a string of stars to print (stars number will represent the "significance level")
/*! \param V 
 *  \param size size of V
 *  \author Flora Jay
 */
string Significance(const double* V, const int size);


//!  Bayesian Confidence Interval with Percentage Boundaries [inf, sup]
/*! \param V 
 *  \param size size of V
 *  \param inf inferior boundary  (should be set to 0.025 for a symetric 95% CI)
 *  \param sup superior boundary  (should be set to 0.975 for a symetric 95% CI)
 *  \author Flora Jay
 */
string CI(const double* V, const int size, const double inf, const double sup);


//! Correlation between values from matrices X and Y
/*! \param X 1st matrix
 *  \param Y 2nd matrix
 *  \author Flora Jay
 */
double Correlation(const DBL_VCTR_2D& X, const DBL_VCTR_2D& Y);

#endif
