#include <cstdlib>
#include <fstream>
#include <iostream>
#include <set>

#include <gsl/gsl_math.h>
#include "IOUtilities.h"
using namespace std;

bool SaveData(const DBL_VCTR_2D& Xtild, const INT_VCTR_3D& X, const string& File)
{
  if (Xtild.size() == 0)
    {
      cerr << "Uninitialized Xtild - SaveData" << endl;
      exit(EXIT_FAILURE);
    }

  if (X.size() == 0)
    {
      cerr << "Uninitialized X - SaveData" << endl;
      exit(EXIT_FAILURE);
    }

  if (Xtild.size() < X.size())
    {
      cerr << "Mismatched Xtild and X - SaveData" << endl;
      exit(EXIT_FAILURE);
    }

  int i, a,l,b;

  ofstream ofs(File.c_str());
  if (!ofs.good())
    {
      cerr << "Cannot open file " << File << " - SaveData" << endl;
      exit(EXIT_FAILURE);
    }

  int M = static_cast<int>(Xtild.size());
  int B = static_cast<int>(Xtild[0].size());
  int N = static_cast<int>(X.size());
  int A = static_cast<int>(X[0].size());
  int L = static_cast<int>(X[0][0].size());

  for (i = 0; i < M; i++)
    {
      for (a = 0; a < A; a++)
        {
	  for (b=1;b < B;b++)
            {
	      ofs << Xtild[i][b] << "\t";
            }
	  if (i < N)
            {
	      for (l = 0; l < L; l++)
                {
		  ofs << X[i][a][l] << "\t";
                }
            }
	  else
            {
	      for (l = 0; l < L; l++)
                {
		  ofs << 0 << "\t";
                }
            }
	  ofs << endl;
        }
    }

  ofs.close();

  return true;
}


void SaveBetaToMyFile(const DBL_VCTR_3D& Beta, const string& File, const int BurnInPeriod){
	int K=(int)Beta.size();
	int P=(int)Beta[0].size();
	int TotalSweep=(int)Beta[0][0].size();
	int KeptSweep = TotalSweep - BurnInPeriod;
	int s,k,p;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveBetaToMyFile" << endl;
		return;
	}
	ofs << "* Beta Posterior Distribution * (Values are sampled after burn-in, with a minimum step of 10 sweeps, and no more than 1000 sweeps of the MCMC are recorded.)" << endl;
	ofs << "Nb.clusters:\t" <<  K <<  "\t Beta.length:\t" << P << endl;

	// Values are sampled each 'step' sweeps, with 'step'>= 10
	// No more than 1000 sweeps are recorded
	if (floor(KeptSweep/10) <= 1000) {
	  for (s=BurnInPeriod; s < TotalSweep; s+=10){

	    for (k=0; k<K; k++){
	      for (p=0; p<P; p++){
		//	cout << Beta[cycle][k][p]<<"\t";
		ofs << Beta[k][p][s] << "\t";
	      }
	    }
	    ofs << endl;
	  }
	} else {
	  int step = floor(KeptSweep/1000);
	  for (s=BurnInPeriod; s < TotalSweep; s+=step){
	    for (k=0; k<K; k++){
	      for (p=0; p<P; p++){
		//	cout << Beta[cycle][k][p]<<"\t";
		ofs << Beta[k][p][s] << "\t";
	      }
	    }
	    ofs << endl;
	  }
	}

	ofs.close();
	cout<<"Beta Posterior Distribution saved in "<<File<<endl;
}


void SaveToMyFileClumppFormat(const DBL_VCTR_2D& V, const string& File, const bool header, const string title)
{
	int i,k;
	int N=static_cast<int>(V.size());
	int K=static_cast<int>(V[0].size());
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveToMyFileClusterFormat" << endl;
		exit(1);
	}

	if (header)
	  {
	    ofs << title << endl;
	    
	    ofs << "Sample\t";
	    for (k = 0; k < K; k++)
	      {
		ofs << "***Cluster  ";
		ofs.width(2);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << k + 1 << "***\t";
	      }
	    ofs << endl;
	  }

	for (i = 0; i < N; i++)
	{
		ofs.width(6);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << i + 1 << ":\t";
		for (k = 0; k < K; k++)
		{
			ofs.width(17);
			ofs.precision(15);
			ofs << fixed;
			ofs << V[i][k] << " ";
		}
		ofs << "\t" << 1 << endl;
	}
	ofs << endl << endl << endl;

}

void WriteInOfstreamClummpFormat(const DBL_VCTR_2D V, ofstream& ofs)
{

	int i,k;
	int N=static_cast<int>(V.size());
	int K=static_cast<int>(V[0].size());

	ofs << "Sample\t";
	for (k = 0; k < K; k++)
	{
		ofs << "***Cluster  ";
		ofs.width(2);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << k + 1 << "***\t";
	}
	ofs << endl;
	for (i = 0; i < N; i++)
	{
		ofs.width(6);
		ofs.fill('0');
		// Make sure the first index is 1 not 0.
		ofs << i + 1 << ":\t";
		for (k = 0; k < K; k++)
		{
			ofs.width(17);
			ofs.precision(15);
			ofs << fixed;
			ofs << V[i][k] << " ";
		}
		ofs << "\t" << 1 << endl;
	}
	ofs << endl << endl << endl;

}


//! Loading Genotype and Covariates Data and other Informations contained in File
bool LoadData(const bool bbase, INT_VCTR_1D& isBase,DBL_VCTR_2D& C, DBL_VCTR_2D& Xtild, INT_VCTR_2D& XQual, INT_VCTR_2D& LevelPopSize, INT_VCTR_1D& nQualLevels, INT_VCTR_3D& X, INT_VCTR_1D& recAllele, int ER, int EC, const string& File, const bool& specialFormat, const int& nbQuantVar, const int& nb_var_regr, const bool& isRecAllele, const int& trendDegree)
{
 
	if (C.size() == 0)
	{
		cerr << "Uninitialized C - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (X.size() == 0)
	{
		cerr << "Uninitialized X - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int i, a, l, b, var;
	int begin_quantitative=1;

	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cerr << "Cannot open file " << File << " - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(C.size()); // N + dummy individuals
	int N = static_cast<int>(X.size());
	int A = static_cast<int>(X[0].size());
	int L = static_cast<int>(X[0][0].size());
	int nbQualVar = static_cast<int>(XQual[0].size()); // Number of Qualitative Variables

	
	double x,y;
	int trendSize,XQuantCol;
	//Degree of trend
	// The intercept is NOT taken into account in trendSize
	if (trendDegree == -1 || nb_var_regr == 0)
	  {
	    trendSize=0;
	  }
	else 
	  {
	    if (nb_var_regr==1)
	      {
		trendSize = trendDegree;
	      }
	    else
	      {
		trendSize = (trendDegree +1)*(trendDegree +2)/2 - 1;
	      }
	  }
		
	if (nbQuantVar==0 && nbQualVar==0 && (trendDegree==-1||trendDegree==0)) // no covariates -> no intercept either
	  {
	    XQuantCol=0;
	  }
	else
	  {
	    // 1 for the intercept + trend + nbQuantVar quantitative covariates 
	    XQuantCol = 1 + trendSize + nbQuantVar;
	  }
	
	DBL_VCTR_2D XQuant; // intercept + quanti + trend
	if (XQuantCol != 0)
	  {
	InitVctr2D(XQuant, N, XQuantCol, 1.0);
	  }

	// Load and discard the extra rows.
	char extraRow[10240];
	
	for (int er = 0; er < ER; er++)
	{
		ifs.getline(extraRow, 10240);
	}

	int recAl = 1;
	if (isRecAllele)
	{
		for (l = 0; l < L; l ++)
		{
			ifs >> recAl;
			recAllele[l] = recAl;
		}
	}

	if (!specialFormat)
	{
		for (i = 0; i < M; i++)
		{
			for (a = 0; a < A; a++)
			{
				// Load and discard the extra columns.
				string discarded;
				for (int ec = 0; ec < EC; ec++)
				{
					ifs >> discarded;
				}

				//Stock base information
				if (bbase)
				{
					ifs >> isBase[i];
				}

				// Qualitative Variables
				if (nbQualVar > 0)
				{
				  for (var = 0; var < nbQualVar; var++)
				    {
				      ifs >> XQual[i][var];
				    }
				}
				
				int begin_coordinates = begin_quantitative + nbQuantVar;
							
				// Quantitative Variable
				for (b = begin_quantitative; b < begin_coordinates; b++)
				{
					ifs >> XQuant[i][b];
				}
				
				// Spatial Coordinates (if given in the datafile)

				if (trendDegree != -1) 
				{  
					ifs >> C[i][0];
					ifs >> C[i][1];

					x = C[i][0];
					y = C[i][1];
										
					if (nb_var_regr == 1)
					{
					  // We keep only y (ignoring x column)
						for (b = begin_coordinates; b < XQuantCol; b ++)
						{
						  XQuant[i][b] = gsl_pow_int(y,b-begin_coordinates+1);
						}
					}

					if (nb_var_regr == 2)
					{
					 
						if (trendDegree > 0)
						{
							XQuant[i][begin_coordinates] = x;
							XQuant[i][begin_coordinates+1] = y;
						
							if (trendDegree > 1)
							{
								XQuant[i][begin_coordinates+2] = x*x;
								XQuant[i][begin_coordinates+3] = y*y;
								XQuant[i][begin_coordinates+4] = x*y;
								if (trendDegree > 2)
								{
									XQuant[i][begin_coordinates+5] = x*x*x;
									XQuant[i][begin_coordinates+6] = y*y*y;
									XQuant[i][begin_coordinates+7] = x*x*y;
									XQuant[i][begin_coordinates+8] = x*y*y;
								}
							}
						}
					}
				}
				// Genotype Data
				if (i < N)
				{
					for (l = 0; l < L; l++)
					{
						ifs >> X[i][a][l];
					}
				}
				else
				{
					for (l = 0; l < L; l++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}
	if (specialFormat)
	{
		for (i = 0; i < M; i ++)
		{
			// Load and discard the extra columns.
			string discarded;
			for (int ec = 0; ec < EC; ec++)
			{
				ifs >> discarded;
			}
			// Stock base information
			if (bbase)
			{
				ifs >> isBase[i];
			}

			// Qualitative Variables
			if (nbQualVar > 0)
			  {
			    for (var = 0; var < nbQualVar; var++)
			      {
				ifs >> XQual[i][var];
			      }
			  }
						
			int begin_coordinates=begin_quantitative + nbQuantVar;
			// Quantitative Variable
			for (b = begin_quantitative; b < begin_coordinates; b++)
			{
				ifs >> XQuant[i][b];
			}

			// Spatial Coordinates (if given in the datafile)
			if (trendDegree != -1) 
			{
				ifs >> C[i][0] >> C[i][1];
				x = C[i][0];
				y = C[i][1];
				
				if (nb_var_regr == 1)
				{
					// We keep only y (ignoring leftmost column)
					for (b = begin_coordinates; b < XQuantCol; b ++)
					{
						XQuant[i][b] = gsl_pow_int(y,b-begin_coordinates+1);
					}
				}
				
				if (nb_var_regr == 2)
				{
					
				  if (trendDegree > 0) 
					{
						XQuant[i][begin_coordinates] = x;
						XQuant[i][begin_coordinates+1] = y;
						if (trendDegree > 1) 
						{
							XQuant[i][begin_coordinates+2] = x*x;
							XQuant[i][begin_coordinates+3] = y*y;
							XQuant[i][begin_coordinates+4] = x*y;
							
							if (trendDegree > 2) 
							{
								XQuant[i][begin_coordinates+5] = x*x*x;
								XQuant[i][begin_coordinates+7] = y*y*y;
								XQuant[i][begin_coordinates+8] = x*x*y;
								XQuant[i][begin_coordinates+9] = x*y*y;
							}
						}
					}
				}
			}
			

			//Genotype Data
			if (i < N)
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> X[i][a][l];
					}
				}
			}
			else
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}

	ifs.close();
	

	if (nbQualVar <= 0) 
	  {
	    Xtild = XQuant;
	    return true;
	  }
	else
	  {
	    bool bQual = TransformQualData(XQual, XQuant, Xtild, nQualLevels, LevelPopSize);
	    return bQual;
	  }
	
}


bool TransformQualData(INT_VCTR_2D& XQual, DBL_VCTR_2D& XQuant, DBL_VCTR_2D& Xtild, INT_VCTR_1D& nQualLevels, INT_VCTR_2D& LevelPopSize) 
{
  
  int N =  static_cast<int>(XQuant.size());
  int nbQualVar =  static_cast<int>(XQual[0].size());
  int nbQuantColumns  =  static_cast<int>(XQuant[0].size()); // contains 1 column for intercept
  int nbQualColumns = 0;

  int begin_quantitative,nextCol,currCol;
  // loop index
  int var,i,b,lev;

  vector< set< int > > levelS;
  set< int > emptyS;

  for (var = 0; var < nbQualVar; var++)
    {
      levelS.push_back(emptyS);
    }
  for (var = 0; var < nbQualVar; var++)
    {
      for (i = 0; i < N; i++)
	{
	  levelS[var].insert(XQual[i][var]);
	}
      nQualLevels[var] = static_cast<int> (levelS[var].size());
      nbQualColumns +=  static_cast<int> (levelS[var].size()) - 1;
    }

 

  InitVctr2D(LevelPopSize,nbQualVar,nQualLevels,0);
  InitVctr2D(Xtild,N, nbQualColumns + nbQuantColumns, 0.0);
  
  for (i = 0; i < N; i++)
    {
      Xtild[i][0]=1.0; //Intercept
     
      nextCol=1;
      currCol=1;
      
      for (var = 0; var < nbQualVar; var++)
	{
	  nextCol += nQualLevels[var] - 1;
	  set< int >::const_iterator csiB = levelS[var].begin();
	  set< int >::const_iterator csiE = levelS[var].end();
	  set< int >::const_iterator csi,it;

	  
	  it=levelS[var].find(XQual[i][var]);
	  if (it == csiB)
	    {
	      LevelPopSize[var][0]++;
	      for (b = currCol; b < nextCol; b++)
		{
		  Xtild[i][b]=-1;
		}
	      XQual[i][var]=0; 
	    }
	  else
	    {
	      lev = distance(csiB,it); //should give an index ?
	      LevelPopSize[var][lev]++;
	      Xtild[i][currCol+lev-1]=1; //other columns stay at 0.0
	      XQual[i][var]=lev;
	    }

	  currCol = nextCol;
	}
      
      begin_quantitative = nextCol;
      for (int b = begin_quantitative; b <  static_cast<int> (Xtild[0].size()) ; b++)
	{
	  Xtild[i][b]=XQuant[i][b-begin_quantitative+1]; //XQuant[][1] corresponds to Intercept -> not needed 
	}
    }
 
 
 return true;
 
}


/*
bool LoadData(const bool bbase,DBL_VCTR_2D& C, DBL_VCTR_2D& Xtild, INT_VCTR_2D& QualVar, INT_VCTR_2D& LevelPopSize, INT_VCTR_3D& X, INT_VCTR_1D& recAllele, int ER, int EC, const string& File, const bool& specialFormat, const int& nbQuantVar, const int& nb_var_regr, const bool& isRecAllele)
{
  if (Xtild.size() == 0)
    {
      cerr << "Uninitialized Xtild - LoadData" << endl;
      exit(EXIT_FAILURE);
    }

  if (C.size() == 0)
    {
      cerr << "Uninitialized C - LoadData" << endl;
      exit(EXIT_FAILURE);
    }

  if (X.size() == 0)
    {
      cerr << "Uninitialized X - LoadData" << endl;
      exit(EXIT_FAILURE);
    }

  if (Xtild.size() < X.size())
    {
      cerr << "Mismatched Xtild and X - LoadData" << endl;
      exit(EXIT_FAILURE);
    }

  int i, a, l, b, var;
  int begin_quantitative=1;

  ifstream ifs(File.c_str());
  if (!ifs.good())
    {
      cerr << "Cannot open file " << File << " - LoadData" << endl;
      exit(EXIT_FAILURE);
    }

  int M = static_cast<int>(Xtild.size());
  int B = static_cast<int>(Xtild[0].size());
  int N = static_cast<int>(X.size());
  int A = static_cast<int>(X[0].size());
  int L = static_cast<int>(X[0][0].size());
  // Number of Levels of the Qualitative Variable
  int nbQualVar = static_cast<int>(QualVar.size());
  int nLevels, nextCol, currCol; //temp variable

  double x,y;
  int nbDigitMax = 0;

  // Load and discard the extra rows.
  char extraRow[10240];
  char testChar;
  for (int er = 0; er < ER; er++)
    {
      ifs.getline(extraRow, 10240);
    }

  int recAl = 1;
  if (isRecAllele)
    {
      for (l = 0; l < L; l ++)
        {
	  ifs >> recAl;
	  recAllele[l] = recAl;
        }
    }

  if (!specialFormat)
    {
      for (i = 0; i < M; i++)
        {
	  for (a = 0; a < A; a++)
            {
	      // Load and discard the extra columns.
	      string discarded;
	      for (int ec = 0; ec < EC; ec++)
                {
		  ifs >> discarded;
                }

	      //Discard base information
	      if (bbase)
                {
		  ifs >> discarded;
                }

	      // Qualitative Variables
	      if (nbQualVar > 0)
		{
		  nextCol=1;
		  currCol=1;
		  for (var = 0; var < nbQualVar; var++)
		    {
		      ifs >> QualVar[var][i];
		      QualVar[var][i]--;     // User Writes Classes from 1 to nbQvar corresponding to Labels from 0 to (nbQvar-1) in the algorithm
		      if (a==0) // count only once
			{
			  LevelPopSize[var][QualVar[var][i]]++;      // Computes the Size of Each Class

			  // 1 Qual. Var. with n Levels corresponds to n-1 Columns of the Environmental Data Matrix (cf Saporta p.398)
			  // The First column of Xtild is ALWAYS set to 1 (corrsponding to Intercept in th Regression)
			  // Level 0 -> all columns set to -1
			  // Level i!=0  -> all columns set to 0, the ith, set to 1
			  nLevels = static_cast<int>(LevelPopSize[var].size());
			  nextCol += nLevels-1;
							
			  if (QualVar[var][i] == 0)
			    {
			      for (b=currCol; b < nextCol; b++)
				{
				  Xtild[i][b]=-1;
				}
			    }
			  else
			    {
			      for (b=currCol; b < nextCol; b++)
				{
				  if ( (b-currCol+1) != QualVar[var][i])
				    {
				      Xtild[i][b]=0;
				    }
				  else
				    {
				      Xtild[i][b]=1;
				    }
				}
			    }
			  currCol = nextCol;
			}
		    }
		  if (a==0) begin_quantitative = nextCol;          // (if a>0, nextCol is not upadated anymore)
		}
				

              
	      // Quantitative Variable
	      int begin_coordinates=begin_quantitative + nbQuantVar;
	      for (b = begin_quantitative; b < begin_coordinates; b++)
                {
		  ifs >> Xtild[i][b];
                }
                
	      // Coordinates 
               
	      ifs >> C[i][0];
	      ifs >> C[i][1];

	      x = C[i][0];
	      y = C[i][1];
                
	      if (nb_var_regr == 1)
                {
		  // We keep only y (ignoring x column)
		  for (b = begin_coordinates; b < B; b ++)
                    {
		      Xtild[i][b] = gsl_pow_int(y,b-begin_coordinates+1);
                    }
                }

	      if (nb_var_regr == 2)
                {
		  int trendSize = B - begin_coordinates;
		  //In any case, trendSize is one of 0,2,5,9
		  if (trendSize > 0)
                    {
		      Xtild[i][begin_coordinates] = x;
		      Xtild[i][begin_coordinates+1] = y;
		      if (trendSize > 2)
                        {
			  Xtild[i][begin_coordinates+2] = x*x;
			  Xtild[i][begin_coordinates+3] = y*y;
			  Xtild[i][begin_coordinates+4] = x*y;

			  if (trendSize > 5)
                            {
			      Xtild[i][begin_coordinates+5] = x*x*x;
			      Xtild[i][begin_coordinates+6] = y*y*y;
			      Xtild[i][begin_coordinates+7] = x*x*y;
			      Xtild[i][begin_coordinates+8] = x*y*y;
                            }
                        }
                    }
                }

	      // Genotype Data
	      if (i < N)
                {
		  for (l = 0; l < L; l++)
                    {
		      ifs >> X[i][a][l];
                    }
                }
	      else
                {
		  for (l = 0; l < L; l++)
                    {
		      ifs >> discarded;
                    }
                }
            }
        }
    }
  if (specialFormat)
    
{
		for (i = 0; i < M; i ++)
		{
			// Load and discard the extra columns.
			string discarded;
			for (int ec = 0; ec < EC; ec++)
			{
				ifs >> discarded;
			}
			// Stock base information
			if (bbase)
			{
				ifs >> discarded;
			}

			// Qualitative Variables
			if (nbQualVar > 0)
			{
				nextCol=1;
				currCol=1;
				for (var = 0; var < nbQualVar; var++)
				{
					ifs >> QualVar[var][i];
					QualVar[var][i]--;     // User Writes Classes from 1 to nbQvar corresponding to Labels from 0 to (nbQvar-1) in the algorithm
					
					LevelPopSize[var][QualVar[var][i]]++;      // Computes the Size of Each Class

					// 1 Qual. Var. with n Levels corresponds to n-1 Columns of the Environmental Data Matrix (cf Saporta p.398)
					// The First column of Xtild is ALWAYS set to 1 (corrsponding to Intercept in th Regression)
					// Level 0 -> all columns set to -1
					// Level i!=0  -> all columns set to 0, the ith, set to 1
					nLevels = static_cast<int>(LevelPopSize[var].size());
					nextCol += nLevels-1;
					
					if (QualVar[var][i] == 0)
					{
						for (b=currCol; b < nextCol; b++)
						{
							Xtild[i][b]=-1;
						}
					}
					else
					{
						for (b=currCol; b < nextCol; b++)
						{
							if ( (b-currCol+1) != QualVar[var][i])
							{
								Xtild[i][b]=0;
							}
							else
							{
								Xtild[i][b]=1;
							}
						}
					}
					currCol = nextCol;
				}
				begin_quantitative = nextCol;
			}
			
			int begin_coordinates=begin_quantitative + nbQuantVar;
			// Quantitative Variable
			for (b = begin_quantitative; b < begin_coordinates; b++)
			{
				ifs >> Xtild[i][b];
			}

			// Spatial Coordinates are always given in the datafile for tess or pops
		
				ifs >> C[i][0] >> C[i][1];
				x = C[i][0];
				y = C[i][1];
				
				if (nb_var_regr == 1)
				{
					// We keep only y (ignoring leftmost column)
					for (b = begin_coordinates; b < B; b ++)
					{
						Xtild[i][b] = gsl_pow_int(y,b-begin_coordinates+1);
					}
				}
				
				if (nb_var_regr == 2)
				{
				 int trendSize = B - begin_coordinates;
				 //In any case, trendSize is one of 0,2,5,9	
				  if (trendSize > 0) 
					{
						Xtild[i][begin_coordinates] = x;
						Xtild[i][begin_coordinates+1] = y;
						if (trendSize > 2)
						{
							Xtild[i][begin_coordinates+2] = x*x;
							Xtild[i][begin_coordinates+3] = y*y;
							Xtild[i][begin_coordinates+4] = x*y;
							
							if (trendSize > 5) 
							{
								Xtild[i][begin_coordinates+5] = x*x*x;
								Xtild[i][begin_coordinates+7] = y*y*y;
								Xtild[i][begin_coordinates+8] = x*x*y;
								Xtild[i][begin_coordinates+9] = x*y*y;
							}
						}
					}
				}
		
			

			//Genotype Data
			if (i < N)
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> X[i][a][l];
					}
				}
			}
			else
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}
  ifs.close();

  return true;
}
*/
bool SaveCF(const DBL_VCTR_1D& sigma, const DBL_VCTR_2D& beta, const DBL_VCTR_2D& alpha, const DBL_VCTR_2D& q, const INT_VCTR_3D& zadmix, const DBL_VCTR_3D& p, const string& File)
{

  if (beta.size() == 0)
    {
      cerr << "Uninitialized beta - SaveCF" << endl;
      return false;
    }

  if (q.size() == 0)
    {
      cerr << "Uninitialized Q - SaveCF" << endl;
      return false;
    }

  if (zadmix.size() == 0)
    {
      cerr << "Uninitialized zadmix - SaveCF" << endl;
      return false;
    }

  if (p.size() == 0)
    {
      cerr << "Uninitialized p - SaveCF" << endl;
      return false;
    }

  int i, k, l, j, b,a;

  ofstream ofs(File.c_str());
  if (!ofs.good())
    {
      cerr << "Cannot open file " << File << " - SaveCF" << endl;
      return false;
    }

  // K, L, J
  int K = static_cast<int>(p.size());
  int L = static_cast<int>(p[0].size());
  INT_VCTR_1D J(L, 0);
  for (l = 0; l < L; l++)
    {
      J[l] = static_cast<int>(p[0][l].size());
    }

  ofs << K << ' ';
  ofs << L << ' ';
  for (l = 0; l < L; l++)
    {
      ofs << J[l] << ' ';
    }

  // N,A
  int N = static_cast<int>(zadmix.size());
  int A = static_cast<int>(zadmix[0].size());
  ofs << N << ' ';

  // beta_length
  int B = static_cast<int>(beta[0].size());

  // p
  for (k = 0; k < K; k++)
    {
      for (l = 0; l < L; l++)
        {
	  for (j = 0; j < J[l]; j++)
            {
	      ofs << p[k][l][j] << ' ';
            }
        }
    }

  // zadmix
  for (i = 0; i < N; i++)
    {
      for (a=0;a<A;a++)
        {
	  for (l=0;l<L;l++)
            {
	      ofs << zadmix[i][a][l] << ' ';
            }
        }
    }

  //q
  for (i=0;i<N;i++)
    {
      for (k=0;k<K;k++)
        {
	  ofs << q[i][k] << ' ';
        }
    }

  //alpha
  for (i=0;i<N;i++)
    {
      for (k=0;k<K;k++)
        {
	  ofs << alpha[i][k] << ' ';
        }
    }

  //beta
  for (k = 0; k < K; k ++)
    {
      for (b = 0; b < B; b ++)
        {
	  ofs << beta[k][b] << ' ';
        }
    }

  //sigma
  for (k = 0; k < K; k ++)
    {
      ofs << sigma[k] << ' ';
    }

  ofs.close();

  return true;
}

bool LoadCF(DBL_VCTR_1D& sigma, DBL_VCTR_2D& beta, DBL_VCTR_2D& alpha, DBL_VCTR_2D& q, INT_VCTR_3D& zadmix, DBL_VCTR_3D& p, const string& File)
{

  if (beta.size() == 0)
    {
      cerr << "Uninitialized beta - LoadCF" << endl;
      return false;
    }

  if (q.size() == 0)
    {
      cerr << "Uninitialized Q - LoadCF" << endl;
      return false;
    }

  if (zadmix.size() == 0)
    {
      cerr << "Uninitialized zadmix - LoadCF" << endl;
      return false;
    }

  if (p.size() == 0)
    {
      cerr << "Uninitialized p - LoadCF" << endl;
      return false;
    }

  int i, k, l, j,a,b;

  // K, L, J
  int K = static_cast<int>(p.size());
  int L = static_cast<int>(p[0].size());
  INT_VCTR_1D J(L, 0);
  for (l = 0; l < L; l++)
    {
      J[l] = static_cast<int>(p[0][l].size());
    }

  // N,A
  int N = static_cast<int>(zadmix.size());
  int A = static_cast<int>(zadmix[0].size());

  //B
  int B = static_cast<int>(beta[0].size());

  ifstream ifs(File.c_str());
  if (!ifs.good())
    {
      cerr << "Cannot open file " << File << " - LoadCF" << endl;
      return false;
    }

  // KF, LF, JF
  int KF;
  int LF;
  INT_VCTR_1D JF;
  ifs >> KF;
  ifs >> LF;
  for (l = 0; l < LF; l++)
    {
      JF.push_back(0);
      ifs >> JF[l];
    }

  // NF
  int NF;
  ifs >> NF;
  if (KF != K || LF != L || JF != J || NF != N)
    {
      ifs.close();

      cerr << "CF file conflicts with command-line options" << " - LoadCF" << endl;
      return false;
    }

  for (k = 0; k < KF; k++)
    {
      for (l = 0; l < LF; l++)
        {
	  for (j = 0; j < JF[l]; j++)
            {
	      ifs >> p[k][l][j];
            }
        }
    }

  // zadmix
  for (i = 0; i < N; i++)
    {
      for (a=0;a<A;a++)
        {
	  for (l=0;l<L;l++)
            {
	      ifs >> zadmix[i][a][l];
            }
        }
    }

  //q
  for (i=0;i<N;i++)
    {
      for (k=0;k<K;k++)
        {
	  ifs >> q[i][k];
        }
    }

  //alpha
  for (i=0;i<N;i++)
    {
      for (k=0;k<K;k++)
        {
	  ifs >> alpha[i][k];
        }
    }

  //beta
  for (k = 0; k < K; k ++)
    {
      for (b = 0; b < B; b ++)
        {
	  ifs >> beta[k][b];
        }
    }

  //sigma
  for (k = 0; k < K; k ++)
    {
      ifs >> sigma[k];
    }

  ifs.close();

  return true;
}

bool SaveML(const int& trendDeg, const double& minDIC,const string& File)
{
  ofstream ofs(File.c_str());
  if (!ofs.good())
    {
      cerr << "Cannot open file " << File << " - SaveML" << endl;
      return false;
    }
  ofs << minDIC << ' ';
  ofs << trendDeg << ' ';
  ofs << "1" << ' ';                            //admixture = true
  ofs.close();
  return true;
}

bool LoadML(int& trendDeg, double& minDIC ,const string& File,bool quiet)
{
  ifstream ifs(File.c_str());
  if (!ifs.good())
    {
      if (!quiet)
        {
	  cerr << "Cannot open file " << File << " - LoadML" << endl;
        }
      return false;
    }

  ifs >> minDIC ;
  ifs >> trendDeg;
  ifs.close();
  return true;
}
