
#ifndef __MATRIX_UTILITIES__
#define __MATRIX_UTILITIES__

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include "VectorUtilities.h"

//matrix = cholesky(mat);
void choldc(DBL_VCTR_2D& matrix, const DBL_VCTR_2D mat);

void buildNeighborhoodMatrix(DBL_VCTR_2D& W, const INT_VCTR_2D& NS);
void buildNeighborhoodMatrix(DBL_VCTR_2D& W, const INT_VCTR_2D& NS, const DBL_VCTR_2D& geo, const double theta);

//Compute eigen values of A, return them sorted in eigenVal
void computeEigenvalues(DBL_VCTR_1D& eigenVal, const DBL_VCTR_2D& A);

//Compute a statistic on the symmetric matrix Matrix and store it in stat. 
//option is one of "m","d","x" for "Mean","meDian","maX"
void computeSymmMatrixStat(double& stat, const DBL_VCTR_2D& Matrix, const string option);

// initialise seulement si v2 n'avait pas encore ete initialise
// sinon copie en ecrivant "par dessus" v2
void CopyVctr(const DBL_VCTR_2D& v1,DBL_VCTR_2D& v2);

void PrintVctr(const DBL_VCTR_2D& V);
void PrintVctr(const INT_VCTR_1D& V);
void PrintVctr(const DBL_VCTR_1D& V);

//Multivariate Gaussian variates generator, with means mu, and v_chol = cholesky(covariance matrix). Result is stocked in y. 
void MultivariateGauss(DBL_VCTR_1D& mu, DBL_VCTR_2D& v_chol, DBL_VCTR_1D& Y, gsl_rng*& rng);


//A%*%B (where B has only one colonn)
void Mult(DBL_VCTR_1D& Res, const DBL_VCTR_2D& A, const DBL_VCTR_1D& B);
//A%*%B
void Mult(DBL_VCTR_2D& Res, const DBL_VCTR_2D& A, const DBL_VCTR_2D& B);
//A%*%t(B)
void Mult_t(DBL_VCTR_2D& Res, const DBL_VCTR_2D& A, const DBL_VCTR_2D& B);
//t(A)%*%B
void Mult_t_left(DBL_VCTR_2D& Res, const DBL_VCTR_2D& A, const DBL_VCTR_2D& B);
// result = constant*mat
void Mult_byConstant(DBL_VCTR_2D& result, const DBL_VCTR_2D& mat, const double constant);

// result = mat1 + mat2
void Sum(DBL_VCTR_2D& result, const DBL_VCTR_2D& mat1, const DBL_VCTR_2D& mat2);

// result = mat1 - mat2
void Diff(DBL_VCTR_2D& result, const DBL_VCTR_2D& mat1, const DBL_VCTR_2D& mat2);

//result = t(mat1) - (mat2)
void Diff_t_left(DBL_VCTR_2D& result, const DBL_VCTR_2D& mat1, const DBL_VCTR_2D& mat2);
		
		
//search for max(W[i,-k],0)
double FindBound(const DBL_VCTR_1D& Wi, int k);

void InverseWithChol(DBL_VCTR_2D& res, const DBL_VCTR_2D& A);
void Inverse(DBL_VCTR_2D& res, const DBL_VCTR_2D& A);
void Transpose(DBL_VCTR_2D& res, const DBL_VCTR_2D& A);



#endif //__MATRIX_UTILITIES__

