#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <sstream>

#include "StatUtilities.h"


// return a string of stars to print (stars number will represent the "significance level")
string Significance(const double* V, const int size) {

  if ((0>=quantile(V,size,.05)) && (0<=quantile(V,size,.95))) {
    //cout << quantile(V,size,.05) << " to " << quantile(V,size,.95)<< endl;
    return "no";  //0 in IC 90%
  } else if ((0>=quantile(V,size,.025)) && (0<=quantile(V,size,.975))) {
    //cout << quantile(V,size,.025) << " to " << quantile(V,size,.975)<< endl;
    return "*";  //0 in IC 95 % but not in IC 90%
  } else if ((0>=quantile(V,size,.005)) && (0<=quantile(V,size,.995))) {
    //cout << quantile(V,size,.005) << " to " << quantile(V,size,.995)<< endl;
    return "**";  //0 in IC 99% but not in IC 95%
  } else if ((0>=quantile(V,size,.0005)) && (0<=quantile(V,size,.9995))) {
    //cout << quantile(V,size,.0005) << " to " << quantile(V,size,.9995)<< endl;
    return "***";  //0 in IC 99.9% but not in IC 99%
  } else {
    //cout << "0 not in " << quantile(V,size,.0005) << " to " << quantile(V,size,.9995)<< endl;
    return "****";  //0 not in IC 99.9%
  }

}

// return confidence interval
string CI(const double* V, const int size, const double inf, const double sup) {
  stringstream infCI,supCI;
  infCI << quantile(V,size,inf);
  supCI << quantile(V,size,sup);
  string stringCI = "[ " + infCI.str() + " , " + supCI.str() + " ]";
  return stringCI;

}

double Correlation(const DBL_VCTR_2D& X, const DBL_VCTR_2D& Y) {
  
  // since the sum of each row is 1
  // the average value of a matrix is 1*N/(k*N) ie 1/K
  if ( (X.size()!=Y.size()) ||(X[0].size()!=Y[0].size()) ){
    cout<<"Correlation: Matrices have inadequate dimensions to be multiplied. Exiting..."<<endl;
    exit(1);
  }

  int N = static_cast<int>(X.size());
  int K = static_cast<int>(X[0].size());
  double propMean = N/K; // mean*sampleSize
  double cov = 0; // cov = N*cov(X,Y)
  double varX = 0;
  double varY = 0;
  int i,k;

  for (i=0; i <N; i++) {
    for (k=0; k<K; k++) {
      cov += X[i][k]*Y[i][k];
      varX += X[i][k]*X[i][k];
      varY += Y[i][k]*Y[i][k];
    }
  }
  cov -= propMean; 
  varX -= propMean; 
  varY -= propMean; 
  
   return(cov / (sqrt(varX)*sqrt(varY)));
  
}
