#ifndef __STAT_UTILITIES__
#define __STAT_UTILITIES__

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include "VectorUtilities.h"
#include "ranker.h"




/********************************************************
 ***                  STATS UTILITIES                 ***
 ********************************************************/

//!  Bayesian Confidence Interval with Percentage Boundaries [inf, sup]
/*! \param V 
 *  \param size size of V
 *  \param inf inferior boundary  (should be set to 0.025 for a symetric 95% CI)
 *  \param sup superior boundary  (should be set to 0.975 for a symetric 95% CI)
 *  \author Flora Jay
 */
string CI(const double* V, const int size, const double inf, const double sup);

//! return a string of stars to print (stars number will represent the "significance level")
/*! \param V 
 *  \param size size of V
 *  \author Flora Jay
 */
string Significance(const double* V, const int size);

//! Correlation between values from matrices X and Y
/*! \param X 1st matrix
 *  \param Y 2nd matrix
 *  \author Flora Jay
 */
double Correlation(const DBL_VCTR_2D& X, const DBL_VCTR_2D& Y);


#endif
